@@ -1,6 +1,6 @@
 #ifndef DNSSECCHECKVERSION_H
 #define DNSSECCHECKVERSION_H
 
-#define DNSSEC_CHECK_VERSION "2.0"
+#define .1"
 
 #endif // DNSSECCHECKVERSION_H
