@@ -17,6 +17,8 @@
 #include <QtNetwork/QNetworkRequest>
 #include <QtNetwork/QNetworkReply>
 #include <QCryptographicHash>
+#include <QUrl>
+#include <QUrlQuery>
 
 #include <qdebug.h>
 
@@ -115,37 +117,37 @@ void MainWindow::addAddress(QString server, int row) {
     label->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
 #endif
 
-    m_results->addWidget(light = new QStatusLight(0, &check_basic_dns, server.toAscii().data(), "DNS", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_basic_dns, server.toLatin1().data(), "DNS", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_basic_tcp, server.toAscii().data(), "TCP", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_basic_tcp, server.toLatin1().data(), "TCP", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_do_bit, server.toAscii().data(), "DO", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_do_bit, server.toLatin1().data(), "DO", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_ad_bit, server.toAscii().data(), "AD", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_ad_bit, server.toLatin1().data(), "AD", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_do_has_rrsigs, server.toAscii().data(), "RRSIG", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_do_has_rrsigs, server.toLatin1().data(), "RRSIG", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_small_edns0, server.toAscii().data(), "EDNS0", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_small_edns0, server.toLatin1().data(), "EDNS0", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_can_get_nsec, server.toAscii().data(), "NSEC", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_can_get_nsec, server.toLatin1().data(), "NSEC", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_can_get_nsec3, server.toAscii().data(), "NSEC3", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_can_get_nsec3, server.toLatin1().data(), "NSEC3", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_can_get_dnskey, server.toAscii().data(), "DNSKEY", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_can_get_dnskey, server.toLatin1().data(), "DNSKEY", row), row, column++);
     m_tests.push_back(light);
 
-    m_results->addWidget(light = new QStatusLight(0, &check_can_get_ds, server.toAscii().data(), "DS", row), row, column++);
+    m_results->addWidget(light = new QStatusLight(0, &check_can_get_ds, server.toLatin1().data(), "DS", row), row, column++);
     m_tests.push_back(light);
 
-    //m_results->addWidget(light = new QStatusLight(0, &check_can_get_signed_dname, server.toAscii().data(), "DNAME", row), row, column++);
+    //m_results->addWidget(light = new QStatusLight(0, &check_can_get_signed_dname, server.toLatin1().data(), "DNAME", row), row, column++);
     //m_tests.push_back(light);
 }
 
@@ -360,10 +362,11 @@ void MainWindow::maybeSubmitResults()
 void MainWindow::submitResults(QString locationDescription)
 {
     QUrl accessURL = resultServerBaseURL;
-    accessURL.addQueryItem("dataVersion", "1");
+    QUrlQuery query;
+    query.addQueryItem("dataVersion", "1");
     int count=0;
     foreach(QString serverAddress, m_serverAddresses) {
-        accessURL.addQueryItem("server" + QString::number(count++),
+        query.addQueryItem("server" + QString::number(count++),
                                QCryptographicHash::hash(serverAddress.toUtf8(), QCryptographicHash::Sha1).toHex());
     }
 
@@ -371,11 +374,13 @@ void MainWindow::submitResults(QString locationDescription)
 
     foreach(QStatusLight *light, m_tests) {
         test = light->test();
-        accessURL.addQueryItem(test->name() + QString::number(light->rowNumber()), test->statusString());
+        query.addQueryItem(test->name() + QString::number(light->rowNumber()), test->statusString());
     }
 
-    accessURL.addQueryItem("locationDescription", locationDescription);
-    accessURL.addQueryItem("DNSSECToolsVersion", DNSSEC_CHECK_VERSION);
+    query.addQueryItem("locationDescription", locationDescription);
+    query.addQueryItem("DNSSECToolsVersion", DNSSEC_CHECK_VERSION);
+
+    accessURL.setQuery(query);
 
     if (!m_manager) {
         m_manager = new QNetworkAccessManager();
