@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
@@ -193,8 +193,8 @@ check_results(val_context_t * context, const char *desc, char * name,
 
     }
 
-    fprintf(stderr, "%s: ****END**** %ld.%ld sec\n", desc, duration.tv_sec,
-            duration.tv_usec);
+    fprintf(stderr, "%s: ****END**** %ld.%ld sec\n", desc,
+            (long) duration.tv_sec, (long) duration.tv_usec);
 
     return err;
 }
@@ -555,13 +555,14 @@ endless_loop(void)
     val_free_validator_state();
 }
 
-void 
+int 
 one_test(val_context_t *context, char *name, int class_h, 
         int type_h, u_int32_t flags, int retvals[], int doprint)
 {
+    int rc;
     struct val_response resp;
     memset(&resp, 0, sizeof(struct val_response));
-    sendquery(context, "Result", name, class_h, type_h, flags, retvals, 1, &resp);
+    rc = sendquery(context, "Result", name, class_h, type_h, flags, retvals, 1, &resp);
     fprintf(stderr, "\n");
 
     // If the print option is present, perform query and validation
@@ -572,6 +573,8 @@ one_test(val_context_t *context, char *name, int class_h,
 
     if (resp.vr_response)
         FREE(resp.vr_response);
+
+    return rc;
 }
 
 #if defined(HAVE_PTHREAD_H) && !defined(VAL_NO_THREADS)
@@ -602,10 +605,12 @@ void *firethread_st(void *param) {
     struct thread_params_st *threadparams = (struct thread_params_st *)param;
     /*child process*/
 
+#if 0
 #ifndef WIN32
     fprintf(stderr, "Start of thread %u\n context=%u\n", 
             (unsigned int)pthread_self(), 
             (unsigned int)threadparams->context);
+#endif
 #endif
 
     do {
@@ -615,9 +620,11 @@ void *firethread_st(void *param) {
             sleep(threadparams->wait);
     }while (threadparams->wait);
     
+#if 0
 #ifndef WIN32
     fprintf(stderr, "End of thread %u\n", 
             (unsigned int)pthread_self());
+#endif
 #endif
 
     return NULL;
@@ -627,10 +634,12 @@ void *firethread_ot(void *param) {
     struct thread_params_ot *threadparams = (struct thread_params_ot *)param;
     /*child process*/
 
+#if 0
 #ifndef WIN32
     fprintf(stderr, "Start of thread %u\n context=%u\n", 
             (unsigned int)pthread_self(), 
             (unsigned int)threadparams->context);
+#endif
 #endif
 
     do {
@@ -641,9 +650,11 @@ void *firethread_ot(void *param) {
             sleep(threadparams->wait);
     }while (threadparams->wait);
 
+#if 0
 #ifndef WIN32
     fprintf(stderr, "End of thread %u\n", 
             (unsigned int)pthread_self());
+#endif
 #endif
     
     return NULL;
@@ -939,7 +950,7 @@ main(int argc, char *argv[])
     } else {
 #endif /* VAL_NO_THREADS */
         do { /* endless loop */
-            one_test(context, domain_name, class_h, type_h, flags, retvals,
+            rc = one_test(context, domain_name, class_h, type_h, flags, retvals,
                      doprint);
 
             if (wait)
