@@ -481,6 +481,9 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
 {
     int i, rc, run = 0;
     struct val_response resp;
+#ifdef VAL_FD_LEAK_TEST
+    int startfd;
+#endif
 
     for (i = tcs;
          curr_test != NULL && curr_test->desc != NULL && i <= tce;
@@ -488,6 +491,15 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
 
         memset(&resp, 0, sizeof(resp));
 
+#ifdef VAL_FD_LEAK_TEST
+        startfd = socket(AF_INET,SOCK_DGRAM,0);
+        if (-1 == startfd) {
+            fprintf(stderr,"\n\nsocket failed\n");
+            break;
+        }
+        close(startfd);
+#endif
+
         ++run;
         fprintf(stderr, "%d: ", ++i);
         rc = sendquery(context, curr_test->desc,
@@ -504,6 +516,17 @@ run_suite(val_context_t *context, testcase *curr_test, int tcs, int tce,
         if (rc)
             ++(*failed);
         fprintf(stderr, "\n");
+
+#ifdef VAL_FD_LEAK_TEST
+        rc = socket(AF_INET,SOCK_DGRAM,0);
+        if (rc > 0) {
+            if (rc != startfd)
+                fprintf(stderr,"\n**** end fd %d != start %d\n", rc, startfd);
+            else
+                fprintf(stderr,"\n**** end fd %d == start %d\n", rc, startfd);
+            close(rc);
+        }
+#endif
     }
 
     return run;
@@ -590,7 +613,7 @@ run_suite_async(val_context_t *context, testsuite *suite, testcase *start_test,
         return 0;
     }
 
-    memset(sstats, 0x00, sizeof(sstats));
+    memset(sstats, 0x00, sizeof(*sstats));
     i = tcs;
     sstats->remaining = tce - tcs + 1;
     sstats->in_flight = 0;
@@ -813,6 +836,8 @@ self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
         while(NULL != suite) {
             rc = run_test_suite(context, tcs, tce, flags, suite, doprint,
                                 max_in_flight);
+            if (rc)
+                fprintf(stderr, "bad rc %d from run_test_suite\n", rc);
             /** does rc mean anything? */
             suite = suite->next;
         }
@@ -833,6 +858,9 @@ self_test(val_context_t *context, int tcs, int tce, u_int32_t flags,
             else {
                 rc = run_test_suite(context, tcs, tce, flags, suite, doprint,
                                     max_in_flight);
+                if (rc)
+                    fprintf(stderr, "bad rc %d from run_test_suite %s\n",
+                            rc, name);
                 /** does rc mean anything? */
             }
             
