@@ -57,11 +57,18 @@
 #ifndef RESOLVER_H
 #define RESOLVER_H
 
-#include <sys/types.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef __MINGW32__
 #include <winsock2.h>
 #endif
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 
 #ifdef __cplusplus
 extern          "C" {
@@ -117,9 +124,15 @@ struct timeval;
 #ifndef NS_MAXCDNAME
 #define NS_MAXCDNAME    255     /* maximum compressed domain name */
 #endif
+
+#define TSIG_ALG_HMAC_MD5_STR "hmac-md5.sig-alg.reg.int"
+#define TSIG_ALG_HMAC_SHA1_STR "hmac-sha1"
+#define TSIG_ALG_HMAC_SHA256_STR "hmac-sha256"
+#define TSIG_FUDGE_DEFAULT  300
+
 struct name_server {
     unsigned char  ns_name_n[NS_MAXCDNAME];
-    void           *ns_tsig;
+    struct ns_tsig *ns_tsig;
     unsigned int   ns_security_options;
     unsigned int   ns_status;
     unsigned long  ns_options;
@@ -146,6 +159,9 @@ struct name_server {
 #define SR_QUERY_RECURSE            0x00000002 
 #define SR_QUERY_SET_DO             0x00000004 
 #define SR_QUERY_SET_CD             0x00000008
+#define SR_QUERY_NOREC              0x00000010
+#define SR_QUERY_IPV4_ONLY          0x00000020
+#define SR_QUERY_IPV6_ONLY          0x00000040
 #define SR_QUERY_VALIDATING_STUB_FLAGS  (SR_QUERY_SET_DO | SR_QUERY_SET_CD) 
 #define SR_QUERY_DEFAULT                (SR_QUERY_RECURSE) 
 
@@ -216,7 +232,8 @@ int             res_gettimeofday_buf(char *buf, size_t bufsize);
 struct sockaddr_storage **create_nsaddr_array(int num_addrs);
 struct name_server *create_name_server(void);
 struct name_server *parse_name_server(const char *cp,
-                                      const char *name_n);
+                                      const char *name_n,
+                                      unsigned long options);
 int             clone_ns(struct name_server **cloned_ns,
                          struct name_server *ns);
 int             clone_ns_list(struct name_server **ns_list,
@@ -270,10 +287,6 @@ res_async_query_handle(struct expected_arrival *ea, int *handled, fd_set *fds);
 void
 res_async_query_free(struct expected_arrival *ea);
 
-int
-res_io_check_one(struct expected_arrival *ea, struct timeval *next_evt,
-                 struct timeval *now);
-
 int
 res_io_check_ea_list(struct expected_arrival *ea, struct timeval *next_evt,
                      struct timeval *now, int *net_change, int *active);
@@ -282,9 +295,6 @@ int
 res_io_get_a_response(struct expected_arrival *ea_list, unsigned char **answer,
                       size_t *answer_length, struct name_server **respondent);
 
-void
-res_io_cancel_remaining_attempts(struct expected_arrival *ea);
-
 void
 res_io_cancel_all_remaining_attempts(struct expected_arrival *ea);
 
@@ -312,6 +322,11 @@ res_async_ea_isset(struct expected_arrival *ea, fd_set *fds);
 
 void res_switch_all_to_tcp_tid(int trans_id);
 
+/*
+ * TSIG interface
+ */
+int res_set_ns_tsig(struct name_server *ns, char *tsig);
+
 /*
  * define timersub macro if not defined. Set top of this header file
  * for the license for this hunk of code.
