@@ -1,6 +1,6 @@
 
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_ERRORS_H
@@ -146,7 +146,7 @@ extern          "C" {
 #define VAL_AC_TRUST_POINT (VAL_AC_LAST_FAILURE+6)
 #define VAL_AC_SIGNING_KEY (VAL_AC_LAST_FAILURE+7)
 #define VAL_AC_VERIFIED_LINK (VAL_AC_LAST_FAILURE+8) 
-#define VAL_AC_UNKNOWN_ALGORITHM_LINK (VAL_AC_LAST_FAILURE+9)
+#define VAL_AC_UNKNOWN_ALGORITHM_LINK (VAL_AC_LAST_FAILURE+9) /* Obsolete */
 
 
 /*
