@@ -48,6 +48,9 @@
     } while(0)
 
 
+static long     _max_fd = 0;
+static long     _open_sockets = 0;
+
 #define MAX_TRANSACTIONS    128
 static struct expected_arrival *transactions[MAX_TRANSACTIONS] = {
     NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
@@ -76,71 +79,108 @@ static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 /*
  * Find a port in the range 1024 - 65535 
  */
+#define NUM_RND_TRIES 10
 static int
 bind_to_random_source(int af, SOCKET s)
 {   
-    struct sockaddr_storage ea_source;
-    struct sockaddr_in *sa4 = (struct sockaddr_in *) &ea_source;
+    struct sockaddr_in sa4;
 #ifdef VAL_IPV6
-    struct sockaddr_in6 *sa6 = (struct sockaddr_in6 *) &ea_source;
+    struct sockaddr_in6 sa6;
 #endif
     struct sockaddr *sa;
     size_t sock_size;
 
-    u_int16_t next_port, start_port;
-
-    memset(&ea_source, 0, sizeof(struct sockaddr_storage));
+    u_int16_t next_port;
+    int count = 0;
 
     if (af == AF_INET) {
-        sa4->sin_family = AF_INET;
-        sa4->sin_addr.s_addr = htonl(INADDR_ANY);
+        memset(&sa4, 0, sizeof(struct sockaddr_in));
+        memset(&sa4.sin_addr, 0, sizeof(sa4.sin_addr));
+        sa4.sin_family = AF_INET;
+        sa4.sin_addr.s_addr = htonl(INADDR_ANY);
 #ifdef VAL_IPV6
     } else if (af == AF_INET6) {
-        sa6->sin6_family = AF_INET6;
-        /*struct in6_addr anyaddr = IN6ADDR_ANY_INIT*/
-        /*sa6->sin6_addr = IN6ADDR_ANY_INIT*/
-        sa6->sin6_addr = in6addr_any;
+        memset(&sa6, 0, sizeof(struct sockaddr_in6));
+        memset(&sa6.sin6_addr, 0, sizeof(sa6.sin6_addr));
+        sa6.sin6_family = AF_INET6;
+        sa6.sin6_addr = in6addr_any;
 #endif
     } else {
         res_log(NULL,LOG_ERR,"libsres: could not bind to random port for unsupported address family %d", af);
         return 1; /* failure */
     }
 
-    start_port = (libsres_random() % 64512) + 1024;
-    next_port = start_port;
 
     do {
-        memset(&ea_source, 0, sizeof(ea_source));
+        next_port = (libsres_random() % 64512) + 1024;
+
         if (af == AF_INET) {
-            sa4->sin_port = htons(next_port);
-            sa = (struct sockaddr *) sa4;
+            sa4.sin_port = htons(next_port);
+            sa = (struct sockaddr *) &sa4;
             sock_size = sizeof(struct sockaddr_in);
 #ifdef VAL_IPV6
-        } else { /* AF_INET6 */
-            sa6->sin6_port = htons(next_port);
-            sa = (struct sockaddr *) sa6;
+        } else {
+            sa6.sin6_port = htons(next_port);
+            sa = (struct sockaddr *) &sa6;
             sock_size = sizeof(struct sockaddr_in6);
 #endif
-        }
+        } 
 
-        if (0 == bind(s, (const struct sockaddr *)sa, sock_size)) {
+        if (0 == bind(s, sa, sock_size)) {
             //res_log(NULL,LOG_ERR,"libsres: bound to random port %d", next_port);
             return 0; /* success */
-        } else  {
-            /* error */
-            if (next_port == 65535)
-                next_port = 1024;
-            else
-                next_port++;
         }
-    } while (next_port != start_port);
+    } while (count++ < NUM_RND_TRIES);
 
     /* wrapped around and still no ports found */
-    res_log(NULL,LOG_ERR,"libsres: could not bind to random port above %d", start_port);
+    res_log(NULL,LOG_ERR,"libsres: could not bind to random port");
 
     return 1; /* failure */
 }
 
+/*
+ * find the max number of file descriptors for this process
+ */
+static int _init_max_fd(void) {
+    struct rlimit info;
+    int rc;
+    int lim;
+
+#ifndef HAVE_GETRLIMIT
+    /** hmm, cannot get current value. let's just guess. */
+    info.rlim_cur = info.rlim_max = SR_IO_NOFILE_UNKNOWN_SIZE;
+#else
+    rc = getrlimit( RLIMIT_NOFILE, &info);
+    if (rc) {
+        /** hmm, cannot get current value. let's just guess. */
+        info.rlim_cur = info.rlim_max = SR_IO_NOFILE_UNKNOWN_SIZE;
+    }
+#ifdef HAVE_SETRLIMIT
+    else if (info.rlim_cur < info.rlim_max) {
+        int prev = info.rlim_cur;
+        info.rlim_cur = info.rlim_max;
+        rc = setrlimit( RLIMIT_NOFILE, &info);
+        if (rc) /* oh well, we tried. revert to previous limit */
+            info.rlim_cur = prev;
+    }
+#endif /* HAVE_GETRLIMIT */
+#endif /* HAVE_GETRLIMIT */
+    lim = info.rlim_cur - SR_IO_NOFILE_RESERVED;
+    if (lim <= 0)
+        lim = 1;
+    return lim;
+}
+
+long
+res_io_get_max_fd(void) {
+    return _max_fd;
+}
+
+long
+res_io_get_open_sockets(void) {
+    return _open_sockets;
+}
+
 long
 res_get_timeout(struct name_server *ns)
 {
@@ -174,8 +214,10 @@ res_sq_free_expected_arrival(struct expected_arrival **ea)
     if ((*ea)->name != NULL)
         free((*ea)->name);
 #endif
-    if ((*ea)->ea_socket != INVALID_SOCKET)
+    if ((*ea)->ea_socket != INVALID_SOCKET) {
         CLOSESOCK((*ea)->ea_socket);
+        --_open_sockets;
+    }
     if ((*ea)->ea_signed)
         FREE((*ea)->ea_signed);
     if ((*ea)->ea_response)
@@ -266,6 +308,7 @@ res_io_retry_source(struct expected_arrival *ea)
     /* close socket */
     if (ea->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(ea->ea_socket);
+        --_open_sockets;
         ea->ea_socket = INVALID_SOCKET;
     }
 
@@ -286,6 +329,7 @@ res_io_reset_source(struct expected_arrival *ea)
     /* close socket */
     if (ea->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(ea->ea_socket);
+        --_open_sockets;
         ea->ea_socket = INVALID_SOCKET;
     }
 
@@ -306,6 +350,7 @@ res_io_cancel_source(struct expected_arrival *ea)
     /* close socket */
     if (ea->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(ea->ea_socket);
+        --_open_sockets;
         ea->ea_socket = INVALID_SOCKET;
     }
 
@@ -350,15 +395,31 @@ res_io_send(struct expected_arrival *shipit)
     size_t          socket_size;
     size_t          bytes_sent;
     long            delay;
+    struct timeval  timeout;
 
     if (shipit == NULL)
         return SR_IO_INTERNAL_ERROR;
 
+    /** init _max_fd as needed */
+    if (0 == _max_fd) {
+        _max_fd =  _init_max_fd();
+        res_log(NULL, LOG_INFO, "libsres: ""max fd  initialized to %d", _max_fd);
+    }
+
     socket_type = (shipit->ea_using_stream == 1) ? SOCK_STREAM : SOCK_DGRAM;
     socket_proto = (socket_type == SOCK_STREAM) ? IPPROTO_TCP : IPPROTO_UDP;
-    res_log(NULL, LOG_DEBUG, "libsres: ""ea %p SENDING type %d %s", shipit,
+    res_log(NULL, LOG_DEBUG, "libsres: ""ea %p SENDING type %d %s over %s",
+            shipit,
             shipit->ea_ns->ns_address[shipit->ea_which_address]->ss_family,
-            shipit->ea_using_stream ? "stream" : "dgram");
+            shipit->ea_using_stream ? "stream" : "dgram",
+            (socket_proto == IPPROTO_TCP) ? "tcp" : "udp");
+
+    /* don't send too many packets at once. */
+    if (shipit->ea_socket == INVALID_SOCKET && _open_sockets >= _max_fd) {
+        res_log(NULL, LOG_DEBUG, "libsres: ""ea %p too many packets in flight",
+                shipit);
+        return SR_IO_TOO_MANY_TRANS;
+    }
 
     /*
      * If no socket exists for the transfer, create and connect it (TCP
@@ -375,6 +436,7 @@ res_io_send(struct expected_arrival *shipit)
                     errno, strerror(errno));
             return SR_IO_SOCKET_ERROR;
         }
+        ++_open_sockets;
 
         /* Set the source port */
         if (0 != bind_to_random_source(af, shipit->ea_socket)) {
@@ -383,6 +445,19 @@ res_io_send(struct expected_arrival *shipit)
             return SR_IO_SOCKET_ERROR;
         }
 
+        /*
+         * Set the timeout interval
+         */
+        timeout.tv_sec = shipit->ea_ns->ns_retrans;
+        timeout.tv_usec = 0;
+
+        if (setsockopt (shipit->ea_socket, SOL_SOCKET, SO_SNDTIMEO, 
+                    (char *)&timeout, sizeof(timeout)) < 0) {
+            /* error */
+            res_io_retry_source(shipit);
+            return SR_IO_SOCKET_ERROR;
+        }
+
         /*
          * OS X wants the socket size to be sockaddr_in for INET,
          * while Linux is happy with sockaddr_storage. 
@@ -519,34 +594,37 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
                   const u_int16_t class_h, const u_int16_t type_h)
 {
     const static int edns0_fallback[] = { 4096, 1492, 512, 0 };
+    int fallback_max_index = 3; /* This must match the array above */
+
     long             i, old_size;
     struct expected_arrival *temp = ea;
 
     if (!temp || !name)
         return -1;
 
-    if (!server && ea->ea_next) {
+    if (!server && temp->ea_next) {
         res_log(NULL, LOG_DEBUG,
                 "libsres: ""no server specified and more than one ea");
         return -1;
     }
 
     if (server) {
-        for(;temp && temp->ea_ns;temp=temp->ea_next) {
+        for(;temp;temp=temp->ea_next) {
             //res_print_ea(temp);
             /** match name, then look for address */
-            if (namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
+            if (!temp->ea_ns || namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
                 continue;
             if (memcmp(server->ns_address[0],
                        temp->ea_ns->ns_address[temp->ea_which_address],
                        sizeof(*server->ns_address[0])) == 0)
                 break;
         }
-        if (!temp) {
-            res_log(NULL, LOG_DEBUG, "libsres: "
-                    "no matching server found for fallback");
-            return -1;
-        }
+    }
+
+    if (!temp || !temp->ea_ns) {
+        res_log(NULL, LOG_DEBUG, "libsres: "
+                "no matching server found for fallback");
+        return -1;
     }
 
     /** even if there is a smaller size to fall back to, no attempts left */
@@ -563,7 +641,7 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     old_size = temp->ea_ns->ns_edns0_size;
     if ((temp->ea_ns->ns_options & SR_QUERY_VALIDATING_STUB_FLAGS) && 
         (temp->ea_ns->ns_edns0_size > 0)) {
-        for (i = 0; i < sizeof(edns0_fallback); i++) {
+        for (i = 0; i < fallback_max_index; i++) {
             if (temp->ea_ns->ns_edns0_size > edns0_fallback[i]) {
                 /* try using a lower edns0 value */
                 temp->ea_ns->ns_edns0_size = edns0_fallback[i];
@@ -612,8 +690,10 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     }
 
     /** close socket so retry uses different port */
-    if (temp->ea_socket != INVALID_SOCKET)
+    if (temp->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(temp->ea_socket);
+        --_open_sockets;
+    }
     temp->ea_socket = INVALID_SOCKET;
 
     res_log(NULL, LOG_INFO, "libsres: "
@@ -637,6 +717,7 @@ res_io_next_address(struct expected_arrival *ea,
          */
         if (ea->ea_socket != INVALID_SOCKET) {
             CLOSESOCK (ea->ea_socket);
+            --_open_sockets;
             ea->ea_socket = INVALID_SOCKET;
         }
         ea->ea_which_address++;
@@ -663,7 +744,7 @@ res_io_next_address(struct expected_arrival *ea,
     res_print_ea(ea);
 }
 /*
- * net_change : optional pointer for returning the next change in the
+ * net_change : optional pointer for returning the net change in the
  *              number of open/active sockets.
  *
  * active : optional pointer for returning number of active queries.
@@ -730,7 +811,7 @@ res_io_check_ea_list(struct expected_arrival *ea, struct timeval *next_evt,
                 }
                 else {
                     if (needed_new_socket) {
-                        if (net_change)
+                        if (net_change && ea->ea_socket != INVALID_SOCKET)
                             ++(*net_change);
                     }
                     break; /* from while remaining attempts */
@@ -878,6 +959,7 @@ res_io_deliver(int *transaction_id, u_char * signed_query,
     int             rc;
 
     rc = res_io_queue(transaction_id, signed_query, signed_length, ns, delay);
+    res_log(NULL, LOG_DEBUG, "libsres: "" res_io_queue rc %d", rc);
 
     /*
      * Call the res_io_check routine 
@@ -1143,6 +1225,7 @@ wait_for_res_data(fd_set * pending_desc, struct timeval *closest_event)
             closest_event->tv_sec, closest_event->tv_usec);
     res_io_set_timeout(&timeout, closest_event);
     ready = res_io_select_sockets(pending_desc, &timeout); 
+    res_log(NULL, LOG_DEBUG, "libsres: ""   %d ready", ready);
 	
     // ignore return value from previous function, 
     // will catch this condition when we actually read data
@@ -1210,6 +1293,7 @@ res_io_get_a_response(struct expected_arrival *ea_list, u_char ** answer,
             /** close socket so retry uses different port */
             if (ea_list->ea_socket != INVALID_SOCKET) {
                 CLOSESOCK (ea_list->ea_socket);
+                --_open_sockets;
                 ea_list->ea_socket = INVALID_SOCKET;
             }
             res_print_ea(ea_list);
@@ -1448,6 +1532,7 @@ res_switch_to_tcp(struct expected_arrival *ea)
     ea->ea_using_stream = TRUE;
     if (ea->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(ea->ea_socket);
+        --_open_sockets;
         ea->ea_socket = INVALID_SOCKET;
     }
     ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
@@ -1477,6 +1562,7 @@ res_switch_all_to_tcp(struct expected_arrival *ea)
         ea->ea_using_stream = TRUE;
         if (ea->ea_socket != INVALID_SOCKET) {
             CLOSESOCK(ea->ea_socket);
+            --_open_sockets;
             ea->ea_socket = INVALID_SOCKET;
         }
     }
@@ -1551,8 +1637,10 @@ res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
                 /*
                  * The the query and response ID's/query lines don't match 
                  */
-                res_log(NULL, LOG_WARNING, "libsres: ""dropping response: "
-                        "query and response ID's or q_fields don't match");
+                res_log(NULL, LOG_WARNING, 
+                        "libsres: ""dropping response with rcode=%x : " 
+                        "query and response ID's or query names don't match",
+                        ((HEADER *) arrival->ea_response)->rcode);
                 FREE(arrival->ea_response);
                 arrival->ea_response = NULL;
                 arrival->ea_response_length = 0;
@@ -1861,8 +1949,12 @@ res_async_query_send(const char *name, const u_int16_t type_h,
     struct expected_arrival *head = 
         res_async_query_create(name, type_h, class_h, pref_ns, 0);
 
-    if (NULL != head)
+    if (NULL != head) {
         ret_val = res_io_check_ea_list(head,NULL,NULL,NULL,NULL);
+        res_log(NULL,LOG_DEBUG, "libsres: "" res_io_check_ea_list returned %d",
+                ret_val);
+    } else
+        res_log(NULL,LOG_DEBUG, "libsres: "" *** res_async_query_create returned NULL");
 
     return head;
 }
@@ -1894,6 +1986,9 @@ res_async_query_create(const char *name, const u_int16_t type_h,
      */
     for (ns = ns_list; ns; ns = ns->ns_next) {
 
+        signed_query = NULL;
+        signed_length = 0;
+
         /** create payload */
         ret_val = res_create_query_payload(ns, name, class_h, type_h,
                                            &signed_query, &signed_length);
