@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -8,6 +8,13 @@
  */ 
 #include "validator-internal.h"
 
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
+#ifdef HAVE_IFADDRS_H
+#include <ifaddrs.h>
+#endif
+
 #include "val_support.h"
 #include "val_policy.h"
 #include "val_cache.h"
@@ -60,6 +67,95 @@ pthread_mutex_t ctx_default =  PTHREAD_MUTEX_INITIALIZER;
 #endif
 
 
+/*
+ * check if we have ipv4/ipv6 addresses
+ */
+static void
+_have_addrs(int *have4, int *have6) {
+#ifndef HAVE_GETIFADDRS
+    /* Return availability of both */
+    if (have4)
+        *have4 = 1;
+    if (have6)
+        *have6 = 1;
+#else
+    struct ifaddrs *ifaddr, *ifa;
+    in_addr_t addr;
+    struct in6_addr addr6;
+    int family;
+
+    if (NULL == have4
+#ifdef VAL_IPV6
+       && NULL == have6
+#endif
+       ) {
+        return;
+    }
+
+    val_log(NULL, LOG_INFO, "_have_addrs(): checking for A/AAAA addrs");
+
+    if (have4)
+        *have4 = 0;
+    if (have6)
+        *have6 = 0;
+
+    if (getifaddrs(&ifaddr) == -1) {
+        val_log(NULL, LOG_ERR, "getifaddrs failed");
+        return;
+    }
+
+    /* Walk through linked list, maintaining head pointer so we
+       can free list later */
+
+    for (ifa = ifaddr; ifa != NULL; ifa = ifa->ifa_next) {
+        if (ifa->ifa_addr == NULL)
+            continue;
+
+        family = ifa->ifa_addr->sa_family;
+
+        if (have4 && family == AF_INET && *have4 == 0) {
+            addr = ((struct sockaddr_in *) (ifa->ifa_addr))->sin_addr.s_addr;
+            if ((ifa->ifa_flags & IFF_UP)
+#ifdef IFF_RUNNING
+                && (ifa->ifa_flags & IFF_RUNNING)
+#endif                          /* IFF_RUNNING */
+                && !(ifa->ifa_flags & IFF_LOOPBACK)
+                && addr != INADDR_LOOPBACK) {
+                ++*have4;
+                val_log(NULL, LOG_INFO, "have v4 addr!");
+            }
+        }
+#ifdef VAL_IPV6
+        else if (have6 && family == AF_INET6 && *have6 == 0) {
+            addr6 = ((struct sockaddr_in6 *) (ifa->ifa_addr))->sin6_addr;
+            if ((ifa->ifa_flags & IFF_UP)
+#ifdef IFF_RUNNING
+                && (ifa->ifa_flags & IFF_RUNNING)
+#endif                          /* IFF_RUNNING */
+                && !(ifa->ifa_flags & IFF_LOOPBACK)
+                && !IN6_IS_ADDR_LOOPBACK(&addr6)
+                && !IN6_IS_ADDR_LINKLOCAL(&addr6)
+                ) {
+                ++*have6;
+                val_log(NULL, LOG_INFO, "have v6 addr!");
+            }
+        }
+#endif
+        else
+            continue;
+        if ((NULL == have4 || *have4) 
+#ifdef VAL_IPV6
+             && (NULL == have6 || *have6)
+#endif
+           )
+            break;
+    } /* for ifrp */
+    goto cleanup;
+
+  cleanup:
+    freeifaddrs(ifaddr);
+#endif
+}
 
 /*
  * re-read resolver policy into the context
@@ -189,15 +285,33 @@ err:
  * default_context.
  */
 static int
-val_create_context_internal( char *label, 
+val_create_context_internal( const char *label, 
                              unsigned int flags,
+                             unsigned int polflags,
+                             char *valpol,
+                             char *res_nslist,
                              char *dnsval_conf, 
                              char *resolv_conf, 
                              char *root_conf, 
+                             val_global_opt_t *valpolopt,
                              val_context_t ** newcontext)
 {
     int             retval;
-    char *base_dnsval_conf = NULL;
+    struct policy_overrides *dyn_valpol;
+    val_global_opt_t *dyn_valpolopt;
+    struct name_server *dyn_nslist;
+    struct name_server *ns;
+    struct name_server *ns_tail;
+    char *buf_ptr, *end_ptr;
+    int  line_number;
+    struct policy_fragment *pol_frag;
+    int g_opt_seen;
+    int include_seen;
+    char *resptr;
+    char *resend;
+    char *rescur;
+    char token[TOKEN_MAX];
+    unsigned long ns_options = 0;
 
 #ifdef WIN32 
     if (!wsaInitialized) {
@@ -208,10 +322,104 @@ val_create_context_internal( char *label,
     }
 #endif
 
-
     if (newcontext == NULL)
         return VAL_BAD_ARGUMENT;
 
+    *newcontext = NULL;
+
+    /*
+     * Process any dynamic policy components
+     */ 
+    dyn_valpolopt = NULL;
+    dyn_valpol = NULL;
+    dyn_nslist = NULL;
+
+    if (valpolopt != NULL) {
+        if (VAL_NO_ERROR != 
+                (retval = update_dynamic_gopt(&dyn_valpolopt,
+                                              valpolopt))) {
+            goto err;
+        }
+        if (valpolopt->log_target)
+            dyn_valpolopt->log_target = strdup(valpolopt->log_target);
+    }
+    if (valpol != NULL) {
+        buf_ptr = valpol;
+        end_ptr = valpol + strlen(valpol) + 1;
+        pol_frag = NULL;
+        line_number = 1;
+        g_opt_seen = 0;
+        include_seen = 0;
+        while (VAL_NO_ERROR == (retval =
+                    get_next_policy_fragment(&buf_ptr, end_ptr,
+                                             label,
+                                             &pol_frag,
+                                             &line_number,
+                                             &g_opt_seen,
+                                             &include_seen))) {
+            if (!g_opt_seen && !include_seen) {
+                store_policy_overrides(&dyn_valpol, &pol_frag);
+                pol_frag = NULL;
+            }
+            if (buf_ptr >= end_ptr) {
+                /* done reading policy string */
+                break;
+            }
+        }
+        if (retval != VAL_NO_ERROR) {
+            goto err;
+        }
+    }
+    if (res_nslist != NULL && strcmp(res_nslist, "")) {
+        strncpy(token, res_nslist, sizeof(token));
+        resptr = token;
+        resend = resptr+strlen(token)+1;
+        rescur = resptr;
+        ns_tail = NULL;
+
+        while (rescur < resend) {
+            /* 
+             * parse the string of name servers
+             * into individual name server structures
+             */
+            if ((*rescur == ' ') || (*rescur == '\t') || (*rescur == ';')) {
+                if (rescur == resptr) {
+                    /* read past leading spaces */
+                    rescur++;
+                    resptr = rescur;
+                    continue;
+                }
+            } else if (*rescur != '\0'){
+                rescur++;
+                continue;
+            }
+
+            *rescur = '\0';
+            rescur++;
+ 
+            if (polflags & CTX_DYN_POL_RES_NRD) {
+                ns_options = SR_QUERY_NOREC;
+            }
+            ns = parse_name_server(resptr, NULL, SR_QUERY_NOREC);
+
+            /* Ignore name servers that we don't understand */
+            if (ns != NULL) {
+                if (ns_tail == NULL) {
+                    dyn_nslist = ns;
+                    ns_tail = ns;
+                } else {
+                    ns_tail->ns_next = ns;
+                    ns_tail = ns;
+                }
+            }
+            resptr = rescur;
+        }
+        if (dyn_nslist == NULL) {
+            retval = VAL_CONF_PARSE_ERROR;
+            goto err;
+        }
+    }
+
     LOCK_DEFAULT_CONTEXT();
     /* Check if the request is for the default context, and we have one available */
     /* 
@@ -224,6 +432,29 @@ val_create_context_internal( char *label,
           (the_default_context->g_opt->env_policy == VAL_POL_GOPT_OVERRIDE || 
            the_default_context->g_opt->app_policy == VAL_POL_GOPT_OVERRIDE)))) {
 
+        /* Update the dynamic policies */
+        if (the_default_context->dyn_valpolopt != NULL) {
+            if (the_default_context->dyn_valpolopt->log_target)
+                FREE(the_default_context->dyn_valpolopt->log_target);
+            FREE(the_default_context->dyn_valpolopt);
+        }
+        the_default_context->dyn_valpolopt = dyn_valpolopt;
+        dyn_valpolopt = NULL;
+
+        if (the_default_context->dyn_valpol != NULL) {
+            destroy_valpolovr(&the_default_context->dyn_valpol);
+        }
+        the_default_context->dyn_valpol = dyn_valpol;
+        dyn_valpol = NULL;
+
+        if (the_default_context->dyn_nslist != NULL) {
+            free_name_servers(&the_default_context->dyn_nslist);
+        }
+        the_default_context->dyn_nslist = dyn_nslist;
+        dyn_nslist = NULL;
+
+        the_default_context->dyn_polflags = polflags;
+
         *newcontext = the_default_context;
 
         /* have configuration files changed? */
@@ -232,7 +463,7 @@ val_create_context_internal( char *label,
         UNLOCK_DEFAULT_CONTEXT();
 
         if (VAL_NO_ERROR != retval) {
-            return retval;
+            goto err;
         }
 
 #ifdef VAL_REFCOUNTS
@@ -248,7 +479,8 @@ val_create_context_internal( char *label,
 
     *newcontext = (val_context_t *) MALLOC(sizeof(val_context_t));
     if (*newcontext == NULL) {
-        return VAL_OUT_OF_MEMORY;
+        retval = VAL_OUT_OF_MEMORY;
+        goto err;
     }
     memset(*newcontext, 0, sizeof(val_context_t));
 #ifdef VAL_REFCOUNTS
@@ -259,13 +491,15 @@ val_create_context_internal( char *label,
     if (0 != pthread_rwlock_init(&(*newcontext)->pol_rwlock, NULL)) {
         FREE(*newcontext);
         *newcontext = NULL;
-        return VAL_INTERNAL_ERROR;
+        retval = VAL_INTERNAL_ERROR;
+        goto err;
     }
     if (0 != pthread_mutex_init(&(*newcontext)->ac_lock, NULL)) {
         pthread_rwlock_destroy(&(*newcontext)->pol_rwlock);
         FREE(*newcontext);
         *newcontext = NULL;
-        return VAL_INTERNAL_ERROR;
+        retval = VAL_INTERNAL_ERROR;
+        goto err;
     }
 
 #ifdef HAVE_PTHREAD_H
@@ -274,15 +508,19 @@ val_create_context_internal( char *label,
         pthread_mutex_destroy(&(*newcontext)->ac_lock);
         FREE(*newcontext);
         *newcontext = NULL;
-        return VAL_INTERNAL_ERROR;
+        retval = VAL_INTERNAL_ERROR;
+        goto err;
     }
 #endif
 #endif
 
     if (snprintf
-        ((*newcontext)->id, VAL_CTX_IDLEN - 1, "%u", (u_int)(*newcontext)) < 0)
+        ((*newcontext)->id, VAL_CTX_IDLEN - 1, "%lu", (u_long)(*newcontext)) < 0)
         strcpy((*newcontext)->id, "libval");
 
+    /* check if we have ipv4 and ipv6 addresses */
+    _have_addrs(&(*newcontext)->have_ipv4, &(*newcontext)->have_ipv6);
+
     /* 
      * Set default configuration files 
      */
@@ -293,6 +531,14 @@ val_create_context_internal( char *label,
 
     (*newcontext)->root_ns = NULL; 
     (*newcontext)->nslist = NULL; 
+    (*newcontext)->dyn_polflags = polflags;
+
+    (*newcontext)->dyn_valpolopt = dyn_valpolopt;
+    (*newcontext)->dyn_valpol = dyn_valpol;
+    (*newcontext)->dyn_nslist = dyn_nslist;
+    dyn_valpolopt = NULL;
+    dyn_valpol = NULL;
+    dyn_nslist = NULL;
 
     (*newcontext)->e_pol =
         (policy_entry_t **) MALLOC(MAX_POL_TOKEN * sizeof(policy_entry_t *));
@@ -303,6 +549,22 @@ val_create_context_internal( char *label,
     memset(((*newcontext)->e_pol), 0,
            MAX_POL_TOKEN * sizeof(policy_entry_t *));
    
+    (*newcontext)->val_log_targets = NULL;
+    (*newcontext)->q_list = NULL;
+    (*newcontext)->as_list = NULL;
+    (*newcontext)->def_cflags = 0; 
+    (*newcontext)->def_uflags = flags & VAL_QFLAGS_USERMASK; 
+
+    /*
+     * Read the validator configuration file first. Some of the policy
+     * knobs may affect parsing of resolver and root hints files.
+     */
+    (*newcontext)->base_dnsval_conf = dnsval_conf? strdup(dnsval_conf) : dnsval_conf_get();
+    if ((retval =
+         read_val_config_file(*newcontext, label)) != VAL_NO_ERROR) {
+        goto err;
+    }
+
     /*
      * Read the Root Hints file; has to be read before resolver config file 
      */
@@ -310,9 +572,6 @@ val_create_context_internal( char *label,
         goto err;
     }
 
-    /* set the log targets to NULL */
-    (*newcontext)->val_log_targets = NULL;
-
     /*
      * Read the Resolver configuration file 
      */
@@ -320,42 +579,13 @@ val_create_context_internal( char *label,
         goto err;
     }
 
-    /*
-     * Read the validator configuration file 
-     */
-    (*newcontext)->q_list = NULL;
-    base_dnsval_conf = dnsval_conf? strdup(dnsval_conf) : dnsval_conf_get();
-    if (base_dnsval_conf == NULL) {
-        val_log(*newcontext, LOG_ERR, "val_create_context_with_conf(): No dnsval.conf file configured");
-        retval = VAL_CONF_NOT_FOUND;
-        goto err;
-    }
-
-    /* Add a single node in the dnsval_list structure */
-    if (NULL == ((*newcontext)->dnsval_l = 
-                (struct dnsval_list *) MALLOC (sizeof(struct dnsval_list))))  {
-        FREE(base_dnsval_conf);
-        retval = VAL_OUT_OF_MEMORY;
-        goto err;
-    }
-    (*newcontext)->dnsval_l->dnsval_conf = base_dnsval_conf; 
-    (*newcontext)->dnsval_l->v_timestamp = 0;
-    (*newcontext)->dnsval_l->next = NULL;
-    
-    if ((retval =
-         read_val_config_file(*newcontext, label)) != VAL_NO_ERROR) {
-        goto err;
-    }
-
-    (*newcontext)->def_cflags = 0; 
-    (*newcontext)->def_uflags = flags & VAL_QFLAGS_USERMASK; 
     if ((*newcontext)->val_log_targets != NULL) {
         (*newcontext)->def_cflags |= VAL_QUERY_AC_DETAIL;
     }
 
     val_log(*newcontext, LOG_DEBUG, 
             "val_create_context_with_conf(): Context created with %s %s %s", 
-            (*newcontext)->dnsval_l->dnsval_conf,
+            (*newcontext)->base_dnsval_conf,
             (*newcontext)->resolv_conf,
             (*newcontext)->root_conf);
 
@@ -378,7 +608,17 @@ val_create_context_internal( char *label,
     return VAL_NO_ERROR;
 
 err:
-    val_free_context(*newcontext);
+    if (dyn_valpolopt) {
+        if (dyn_valpolopt->log_target)
+            FREE(dyn_valpolopt->log_target);
+        FREE(dyn_valpolopt);
+    }
+    if (dyn_valpol)
+        destroy_valpolovr(&dyn_valpol);
+    if (dyn_nslist)
+        free_name_servers(&dyn_nslist);
+    if (*newcontext)
+        val_free_context(*newcontext);
     *newcontext = NULL;
     return retval;
 }
@@ -387,14 +627,14 @@ err:
  * Create a context with given configuration files
  */
 int
-val_create_context_with_conf(char *label, 
+val_create_context_with_conf(const char *label, 
                              char *dnsval_conf, 
                              char *resolv_conf, 
                              char *root_conf, 
                              val_context_t ** newcontext)
 {
-    return val_create_context_internal(label, 0,
-                dnsval_conf, resolv_conf, root_conf, newcontext); 
+    return val_create_context_internal(label, 0, 0, NULL, NULL,
+                dnsval_conf, resolv_conf, root_conf, NULL, newcontext); 
 }
 
 /*
@@ -402,7 +642,7 @@ val_create_context_with_conf(char *label,
  * and with the given default query flags
  */
 int
-val_create_context_ex(char *label, 
+val_create_context_ex(const char *label,
                       val_context_opt_t *opt,
                       val_context_t ** newcontext)
 {
@@ -411,8 +651,14 @@ val_create_context_ex(char *label,
         return VAL_BAD_ARGUMENT;
 
     return val_create_context_internal(label, 
-                opt->vc_flags, opt->vc_val_conf, 
-                opt->vc_res_conf, opt->vc_root_conf, 
+                opt->vc_qflags, 
+                opt->vc_polflags, 
+                opt->vc_valpol,
+                opt->vc_nslist,
+                opt->vc_val_conf, 
+                opt->vc_res_conf, 
+                opt->vc_root_conf, 
+                opt->vc_gopt, 
                 newcontext); 
 }
 
@@ -421,11 +667,11 @@ val_create_context_ex(char *label,
  * Create a context with default configuration files
  */
 int
-val_create_context(char *label, 
+val_create_context(const char *label, 
                    val_context_t ** newcontext)
 {
-    return val_create_context_internal(label, 0,
-                NULL, NULL, NULL, newcontext);
+    return val_create_context_internal(label, 0, 0, NULL, 
+                NULL, NULL, NULL, NULL, NULL, newcontext);
 }
 
 /*
@@ -447,10 +693,8 @@ val_create_or_refresh_context(val_context_t *ctx)
     val_context_t *context = NULL;
     int retval;
 
-    /*
-     * Create a default context if one does not exist
-     */
     if (ctx == NULL) {
+        /* return the default context */
         if (VAL_NO_ERROR != (retval = val_create_context(NULL, &context)))
             return NULL;
     } else {
@@ -533,7 +777,19 @@ val_free_context(val_context_t * context)
 
     if (context->root_ns)
         free_name_servers(&context->root_ns);
-    
+
+    if (context->dyn_valpolopt) {
+        if (context->dyn_valpolopt->log_target)
+            FREE(context->dyn_valpolopt->log_target);
+        FREE(context->dyn_valpolopt);
+    }
+
+    if (context->dyn_valpol)
+        destroy_valpolovr(&context->dyn_valpol);
+
+    if (context->dyn_nslist)
+        free_name_servers(&context->dyn_nslist);
+
     destroy_respol(context);
     destroy_valpol(context);
     FREE(context->e_pol);
@@ -543,6 +799,9 @@ val_free_context(val_context_t * context)
         free_query_chain_structure(q);
         q = NULL;
     }
+    if (context->base_dnsval_conf)
+        FREE(context->base_dnsval_conf);
+    
 
     FREE(context);
 }
@@ -613,3 +872,32 @@ val_context_setqflags(val_context_t *context,
 
     return VAL_NO_ERROR;
 }  
+
+int
+val_context_ip4(val_context_t * context)
+{
+    if (context) {
+        /* No IPv4 if we're only configured to use IPv6 */
+        if (context->g_opt && 
+            context->g_opt->proto == VAL_POL_GOPT_PROTO_IPV6)
+            return 0;
+        return context->have_ipv4;
+    }
+
+    return 0;
+}
+
+int
+val_context_ip6(val_context_t * context)
+{
+    if (context) {
+        /* No IPv6 if we're only configured to use IPv4 */
+        if (context->g_opt &&
+            context->g_opt->proto == VAL_POL_GOPT_PROTO_IPV4)
+            return 0;
+        return context->have_ipv6;
+    }
+
+    return 0;
+}
+
