@@ -1,5 +1,5 @@
 /*
- * Copyright 2006-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2006-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 #ifndef VAL_CONTEXT_H
@@ -67,15 +67,15 @@
 
 #endif /*VAL_NO_THREADS*/
 
-int             val_create_context_with_conf(char *label,
+int             val_create_context_with_conf(const char *label,
                                              char *dnsval_conf,
                                              char *resolv_conf,
                                              char *root_conf,
                                              val_context_t ** newcontext);
-int             val_create_context_ex(char *label, 
+int             val_create_context_ex(const char *label, 
                                       val_context_opt_t *opt,
                                       val_context_t ** newcontext);
-int             val_create_context(char *label,
+int             val_create_context(const char *label,
                                    val_context_t ** newcontext);
 val_context_t * val_create_or_refresh_context(val_context_t *ctx);
 void            val_free_context(val_context_t * context);
@@ -83,6 +83,8 @@ int             val_free_validator_state(void);
 int             val_context_setqflags(val_context_t *context,
                                       unsigned char action,
                                       unsigned int flags);
+int             val_context_ip4(val_context_t * context);
+int             val_context_ip6(val_context_t * context);
 
 #ifndef VAL_NO_ASYNC
 /* remove asynchronous status from context async queries list */
