@@ -28,6 +28,7 @@ const char resolv_conf_inline_buf[] = "";
 
 #ifdef ANDROID
 #include <sys/system_properties.h>
+#define atexit(func)
 #endif /* ANDROID */
 
 #define READ_POL_FOR_ZONE(buf_ptr, end_ptr, line_number, endst, retval, err, token)  do {\
@@ -229,8 +230,61 @@ int free_policy_entry(policy_entry_t *pol_entry, int index)
 
 }
 
+static void
+set_global_opt_defaults(val_global_opt_t *gopt)
+{
+    if (gopt == NULL)
+        return;
+
+    gopt->local_is_trusted = 0;
+    gopt->edns0_size = RES_EDNS0_DEFAULT;
+    gopt->env_policy = VAL_POL_GOPT_DISABLE;
+    gopt->app_policy = VAL_POL_GOPT_DISABLE;
+    gopt->log_target = NULL;
+    gopt->closest_ta_only = 0;
+    gopt->rec_fallback = 1;
+    gopt->max_refresh = VAL_POL_GOPT_MAXREFRESH;
+    gopt->proto = VAL_POL_GOPT_PROTO_ANY;
+}
+
+int 
+update_dynamic_gopt(val_global_opt_t **g_new, val_global_opt_t *g)
+{
+    if (g_new == NULL || g == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    if (*g_new == NULL) {
+        *g_new = (val_global_opt_t *) MALLOC (sizeof (val_global_opt_t));
+        if (*g_new == NULL) {
+            return VAL_OUT_OF_MEMORY;
+        }
+        set_global_opt_defaults(*g_new);
+    }
+
+    /* NOTE: We must not update log_target */
+
+    if (g->local_is_trusted != VAL_POL_GOPT_UNSET)
+        (*g_new)->local_is_trusted = g->local_is_trusted;        
+    if (g->edns0_size != VAL_POL_GOPT_UNSET)
+        (*g_new)->edns0_size = g->edns0_size;        
+    if (g->env_policy != VAL_POL_GOPT_UNSET)
+        (*g_new)->env_policy = g->env_policy;        
+    if (g->app_policy != VAL_POL_GOPT_UNSET)
+        (*g_new)->app_policy = g->app_policy;        
+    if (g->closest_ta_only != VAL_POL_GOPT_UNSET)
+        (*g_new)->closest_ta_only = g->closest_ta_only;        
+    if (g->rec_fallback != VAL_POL_GOPT_UNSET)
+        (*g_new)->rec_fallback = g->rec_fallback;        
+    if (g->max_refresh != VAL_POL_GOPT_UNSET)
+        (*g_new)->max_refresh = g->max_refresh;        
+    if (g->proto != VAL_POL_GOPT_UNSET)
+        (*g_new)->proto = g->proto;        
+
+    return VAL_NO_ERROR;
+}
+
 void
-free_global_options(global_opt_t *g)
+free_global_options(val_global_opt_t *g)
 {
     if (g) {
         if (g->log_target)
@@ -273,8 +327,6 @@ parse_trust_anchor(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry,
     int             retval;
     char           *pkstr;
     char           *endptr;
-    val_dnskey_rdata_t *dnskey_rdata;
-    val_ds_rdata_t *ds_rdata;
 
     if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
         (pol_entry == NULL) || (line_number == NULL) || (endst == NULL))
@@ -302,11 +354,10 @@ parse_trust_anchor(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry,
         }
         if (VAL_NO_ERROR !=
             (retval =
-                 val_parse_ds_string(pkstr, strlen(pkstr), &ds_rdata))) {
+                 val_parse_ds_string(pkstr, strlen(pkstr), &ta_pol->ds))) {
             FREE(ta_pol);
             return retval;
         }
-        ta_pol->ds = ds_rdata;
 
     } else {
         if (!strncasecmp(pkstr, DNSKEY_STR, strlen(DNSKEY_STR))) {
@@ -323,11 +374,11 @@ parse_trust_anchor(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry,
         */ 
         if (VAL_NO_ERROR !=
             (retval =
-                 val_parse_dnskey_string(pkstr, strlen(pkstr), &dnskey_rdata))) {
+                 val_parse_dnskey_string(pkstr, strlen(pkstr),
+                     &ta_pol->publickey))) {
             FREE(ta_pol);
             return retval;
         }
-        ta_pol->publickey = dnskey_rdata;
     }
 
     pol_entry->pol = ta_pol;
@@ -357,7 +408,7 @@ free_trust_anchor(policy_entry_t * pol_entry)
 }
 
 /*
- * parse additional data (time in seconds, -1 for ignore) for the clock skew policy 
+ * parse additional data (time in seconds) for the clock skew policy 
  */
 int
 parse_clock_skew(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry, 
@@ -750,9 +801,9 @@ done:
  * obtained.
  */
 int
-check_relevance(char *label, char *scope, int *label_count, int *relevant)
+check_relevance(const char *label, const char *scope, int *label_count, int *relevant)
 {
-    char           *c, *p, *e;
+    const char           *c, *p, *e;
 
     /*
      * sanity check; NULL scope is OK 
@@ -802,7 +853,7 @@ check_relevance(char *label, char *scope, int *label_count, int *relevant)
 
 static int
 parse_local_answer_gopt(char **buf_ptr, char *end_ptr, int *line_number, 
-                        int *endst, global_opt_t *g_opt) 
+                        int *endst, val_global_opt_t *g_opt) 
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -834,7 +885,7 @@ parse_local_answer_gopt(char **buf_ptr, char *end_ptr, int *line_number,
 
 static int
 parse_edns0_size_gopt(char **buf_ptr, char *end_ptr, int *line_number, 
-                      int *endst, global_opt_t *g_opt) 
+                      int *endst, val_global_opt_t *g_opt) 
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -861,7 +912,7 @@ parse_edns0_size_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     
 static int
 parse_enable_disable_gopt(int *type, char **buf_ptr, char *end_ptr, int *line_number,
-                          int *endst, global_opt_t *g_opt)
+                          int *endst, val_global_opt_t *g_opt)
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -896,7 +947,7 @@ parse_enable_disable_gopt(int *type, char **buf_ptr, char *end_ptr, int *line_nu
 
 static int
 parse_log_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
-                      int *endst, global_opt_t *g_opt)
+                      int *endst, val_global_opt_t *g_opt)
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -926,7 +977,7 @@ parse_log_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
 
 static int
 parse_closest_ta_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
-                      int *endst, global_opt_t *g_opt)
+                      int *endst, val_global_opt_t *g_opt)
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -959,7 +1010,7 @@ parse_closest_ta_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
 
 static int
 parse_rec_fallback(char **buf_ptr, char *end_ptr, int *line_number,
-                   int *endst, global_opt_t *g_opt)
+                   int *endst, val_global_opt_t *g_opt)
 {
     char            token[TOKEN_MAX];
     int retval;
@@ -990,9 +1041,71 @@ parse_rec_fallback(char **buf_ptr, char *end_ptr, int *line_number,
     return VAL_NO_ERROR;
 }
 
+static int
+parse_max_refresh_gopt(char **buf_ptr, char *end_ptr, int *line_number, 
+                      int *endst, val_global_opt_t *g_opt) 
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    g_opt->max_refresh = strtol(token, (char **)NULL, 10);
+    
+    return VAL_NO_ERROR;
+}
+
+static int
+parse_proto(char **buf_ptr, char *end_ptr, int *line_number,
+            int *endst, val_global_opt_t *g_opt)
+{
+    char            token[TOKEN_MAX];
+    int retval;
+
+    if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
+        (g_opt == NULL) || (endst == NULL) || (line_number == NULL))
+        return VAL_BAD_ARGUMENT;
+
+    /* read the next token */
+    if (VAL_NO_ERROR != (retval = 
+        val_get_token(buf_ptr, end_ptr, line_number, 
+                      token, sizeof(token), endst,
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
+        return retval;
+    }
+    if ((endst && (strlen(token) == 0)) ||
+        (*buf_ptr >= end_ptr)) { 
+        return VAL_CONF_PARSE_ERROR;
+    }
+
+    if (!strncmp(token, GOPT_PROTO_IPV6_STR, strlen(GOPT_PROTO_IPV6_STR))) {
+        g_opt->proto = VAL_POL_GOPT_PROTO_IPV6;
+    } else if (!strncmp(token, GOPT_PROTO_IPV4_STR, strlen(GOPT_PROTO_IPV4_STR))) {
+        g_opt->proto = VAL_POL_GOPT_PROTO_IPV4;
+    } else if (!strncmp(token, GOPT_PROTO_ANY_STR, strlen(GOPT_PROTO_ANY_STR))) {
+        g_opt->proto = VAL_POL_GOPT_PROTO_ANY;
+    } else {
+        return VAL_CONF_PARSE_ERROR;
+    }
+    return VAL_NO_ERROR;
+}
+
 static int
 get_global_options(char **buf_ptr, char *end_ptr, 
-                   int *line_number, global_opt_t **g_opt) 
+                   int *line_number, val_global_opt_t **g_opt) 
 {
     int endst = 0;
     char            token[TOKEN_MAX];
@@ -1002,17 +1115,10 @@ get_global_options(char **buf_ptr, char *end_ptr,
         (g_opt == NULL) || (line_number == NULL))
         return VAL_BAD_ARGUMENT;
 
-    *g_opt = (global_opt_t *) MALLOC (sizeof (global_opt_t));
+    *g_opt = (val_global_opt_t *) MALLOC (sizeof (val_global_opt_t));
     if (*g_opt == NULL)
         return VAL_OUT_OF_MEMORY;
-    (*g_opt)->local_is_trusted = 0;
-    (*g_opt)->edns0_size = RES_EDNS0_DEFAULT;
-    (*g_opt)->env_policy = VAL_POL_GOPT_DISABLE;
-    (*g_opt)->app_policy = VAL_POL_GOPT_DISABLE;
-    (*g_opt)->log_target = NULL;
-    (*g_opt)->closest_ta_only = 0;
-    (*g_opt)->rec_fallback = 1;
-    
+    set_global_opt_defaults(*g_opt);
     while (!endst) {
         /*
          * read the option type 
@@ -1077,6 +1183,20 @@ get_global_options(char **buf_ptr, char *end_ptr,
                 goto err;
             }
 
+        } else if (!strcmp(token, GOPT_MAX_REFRESH_STR)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_max_refresh_gopt(buf_ptr, end_ptr,
+                                                    line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
+        } else if (!strcmp(token, GOPT_PROTO)) {
+            if (VAL_NO_ERROR != 
+                    (retval = parse_proto(buf_ptr, end_ptr,
+                                          line_number, &endst, *g_opt))) {
+                goto err;
+            }
+
         } else {
             retval = VAL_CONF_PARSE_ERROR;
             goto err;
@@ -1095,8 +1215,8 @@ err:
  * Get the next relevant {label, keyword, data} fragment 
  * from the configuration file file
  */
-static int
-get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
+int
+get_next_policy_fragment(char **buf_ptr, char *end_ptr, const char *scope,
                          struct policy_fragment **pol_frag,
                          int *line_number, int *g_opt_seen, int *include_seen)
 {
@@ -1270,15 +1390,14 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
  * This list is ordered from general to more specific --
  * so "mozilla" < "sendmail" < "browser:mozilla"
  */
-static int
-store_policy_overrides(val_context_t * ctx, 
-                       struct policy_overrides **overrides, 
+int
+store_policy_overrides(struct policy_overrides **overrides, 
                        struct policy_fragment **pfrag)
 {
     struct policy_overrides *cur, *prev, *newp;
     struct policy_list *e;
 
-    if ((ctx == NULL) || (overrides == NULL) || (pfrag == NULL) || (*pfrag == NULL))
+    if ((overrides == NULL) || (pfrag == NULL) || (*pfrag == NULL))
         return VAL_BAD_ARGUMENT;
 
     /*
@@ -1326,16 +1445,12 @@ store_policy_overrides(val_context_t * ctx,
     /*
      * Check if we have an existing entry for this label/policy-type combination 
      */
-    val_log(ctx, LOG_DEBUG, "store_policy_overrides(): Adding policy [%s:%s]", 
-            newp->label, 
-            conf_elem_array[(*pfrag)->index].keyword);
-
     for (e = newp->plist; e; e = e->next) {
         if (e->index == (*pfrag)->index) {
-            val_log(ctx, LOG_WARNING,
-                    "store_policy_overrides(): Duplicate policy definition for [%s:%s]; using initial",
-                    newp->label, conf_elem_array[e->index].keyword);
-            
+            /* 
+             * Duplicate policy definition conf_elem_array[e->index].keyword
+             * Use the first one found
+             */
             free_policy_entry((*pfrag)->pol, (*pfrag)->index);
             (*pfrag)->pol = NULL;
             FREE(*pfrag);
@@ -1361,7 +1476,7 @@ store_policy_overrides(val_context_t * ctx,
     return VAL_NO_ERROR;
 }
 
-static void
+void
 destroy_valpolovr(struct policy_overrides **po)
 {
     struct policy_overrides *cur, *prev;
@@ -1427,7 +1542,7 @@ destroy_valpol(val_context_t * ctx)
 }
 
 
-#ifdef __linux__
+#if defined(__linux__) && !defined(eabi)
 #define getprogname() program_invocation_short_name 
 #endif
 #ifdef solaris2
@@ -1445,12 +1560,12 @@ destroy_valpol(val_context_t * ctx)
 
 static int
 read_next_val_config_file(val_context_t *ctx, 
-                          char **label, 
+                          const char **label, 
                           struct dnsval_list *dnsval_c, 
                           struct dnsval_list *dlist, 
                           struct dnsval_list **added_files,
                           struct policy_overrides **overrides,
-                          global_opt_t **g_opt)
+                          val_global_opt_t **g_opt)
 {
     int    fd = -1;
 #ifdef HAVE_FLOCK
@@ -1469,12 +1584,13 @@ read_next_val_config_file(val_context_t *ctx,
     char *dnsval_filename = NULL;
     struct dnsval_list *dnsval_l;
     int retval = VAL_NO_ERROR;
-    char *next_label;
+    const char *next_label;
     int done;
     char *env = NULL;
 
-    if (ctx == NULL || label == NULL || dnsval_c == NULL || 
-        dlist == NULL || added_files == NULL || 
+    if (ctx == NULL || label == NULL || 
+        dnsval_c == NULL || dlist == NULL ||
+        added_files == NULL || 
         overrides == NULL || g_opt == NULL)
         return VAL_BAD_ARGUMENT;
 
@@ -1483,13 +1599,15 @@ read_next_val_config_file(val_context_t *ctx,
     *added_files = NULL;
 
     next_label = *label;
-    
+   
     fd = open(dnsval_c->dnsval_conf, O_RDONLY);
+
     if (fd < 0) {
         val_log(ctx, LOG_ERR, 
                 "read_next_val_config_file(): Could not open validator conf file for reading: %s",
                 dnsval_c->dnsval_conf);
-        /* check if we have at least one file */
+
+        /* check if we have read at least one file in the past */
         if (dnsval_c != dlist) {
             return VAL_NO_ERROR;
         }
@@ -1581,7 +1699,7 @@ read_next_val_config_file(val_context_t *ctx,
                                              &include_seen))) {
             if (g_opt_seen) {
                 /* next policy fragment contains global options */ 
-                global_opt_t *gt_opt = NULL;
+                val_global_opt_t *gt_opt = NULL;
                 g_opt_seen = 0;
                 if (VAL_NO_ERROR != (retval =
                         get_global_options(&buf_ptr, end_ptr, 
@@ -1631,7 +1749,7 @@ read_next_val_config_file(val_context_t *ctx,
                                     "read_next_val_config_file(): Using policy label from app name: %s",
                                     c_next_label);
                             done = 0;
-                            next_label = (char *)c_next_label;
+                            next_label = (const char *)c_next_label;
                             break;
                         }
                         /* policy does not exist, dont create the impression that we have one */
@@ -1761,7 +1879,7 @@ read_next_val_config_file(val_context_t *ctx,
                 /*
                  * Store this fragment as an override, consume pol_frag 
                  */
-                store_policy_overrides(ctx, overrides, &pol_frag);
+                store_policy_overrides(overrides, &pol_frag);
                 pol_frag = NULL;
             }
             if (buf_ptr >= end_ptr) {
@@ -1809,42 +1927,45 @@ err:
  * Precedence is environment, app and user
  */
 int
-read_val_config_file(val_context_t * ctx, char *scope)
+read_val_config_file(val_context_t * ctx, const char *scope)
 {
-    struct policy_overrides *overrides;
     struct policy_overrides *t;
-    char *base_dnsval_conf = NULL;
-    global_opt_t *g_opt = NULL;
-    struct dnsval_list *dlist = NULL;
     struct dnsval_list *dnsval_c;
     int             retval;
-    char *label;
+    const char *label;
     char *newctxlab;
     struct val_query_chain *q;
     char *logtarget = NULL;
+    val_global_opt_t *g_opt = NULL;
+    struct dnsval_list *dlist = NULL;
+    struct policy_overrides *overrides = NULL;
    
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
+
+    label = scope;
+
+    /*
+     * If our dynamic policies override existing policies
+     * we don't need to read any of the config files 
+     */
+    if ((ctx->dyn_polflags & CTX_DYN_POL_VAL_OVR) &&
+        (ctx->dyn_polflags & CTX_DYN_POL_GLO_OVR))
+        goto skipfileread;
     
-    if (ctx->dnsval_l && ctx->dnsval_l->dnsval_conf)
-        base_dnsval_conf = strdup(ctx->dnsval_l->dnsval_conf);
-    
-    if (base_dnsval_conf == NULL) 
-        return VAL_OUT_OF_MEMORY;
+    if (ctx->base_dnsval_conf == NULL) {
+        goto skipfileread;
+    }
    
     /* create a new head element for the dnsval.conf file list */
     dlist = (struct dnsval_list *) MALLOC (sizeof(struct dnsval_list));
     if (dlist == NULL) {
-        FREE(base_dnsval_conf);
         return VAL_OUT_OF_MEMORY;
     }
-    dlist->dnsval_conf = base_dnsval_conf;
+    dlist->dnsval_conf = strdup(ctx->base_dnsval_conf);
+    dlist->v_timestamp = 0;
     dlist->next = NULL; 
     dnsval_c = dlist;
-    label = scope;
-
-    overrides = NULL;
-    g_opt = NULL;
 
     while(dnsval_c) {
         struct dnsval_list *added_list = NULL;
@@ -1881,6 +2002,7 @@ read_val_config_file(val_context_t * ctx, char *scope)
         dnsval_c = dnsval_c->next;
     }
 
+skipfileread:
     newctxlab = NULL;
     if (label == NULL) {
         /*
@@ -1903,16 +2025,30 @@ read_val_config_file(val_context_t * ctx, char *scope)
 
     destroy_valpol(ctx);
 
-    /* Replace policies */
-    for (t = overrides; t != NULL; t = t->next) {
+    /* process overrides unless we want to override them */
+    if (!(ctx->dyn_polflags & CTX_DYN_POL_VAL_OVR)) {
+        /* Replace policies */
+        for (t = overrides; t != NULL; t = t->next) {
+            struct policy_list *c;
+            for (c = t->plist; c; c = c->next){
+                /* Override elements in e_pol[c->index] with what's in c->pol */
+                STORE_POLICY_ENTRY_IN_LIST(c->pol, ctx->e_pol[c->index]);
+            }
+        }
+    }
+
+    destroy_valpolovr(&overrides);
+
+    /* Apply any dynamic policies */
+    for (t = ctx->dyn_valpol; t != NULL; t = t->next) {
         struct policy_list *c;
         for (c = t->plist; c; c = c->next){
             /* Override elements in e_pol[c->index] with what's in c->pol */
             STORE_POLICY_ENTRY_IN_LIST(c->pol, ctx->e_pol[c->index]);
         }
     }
-    destroy_valpolovr(&overrides);
 
+    /* Process Global options */
     ctx->g_opt = g_opt;
 
     /* free up older log targets */
@@ -1923,17 +2059,30 @@ read_val_config_file(val_context_t * ctx, char *scope)
     }
     ctx->val_log_targets = NULL;
     
-    /* enable logging */
-    if (g_opt && g_opt->log_target) {
-        val_log_add_optarg_to_list(&ctx->val_log_targets, g_opt->log_target, 1);
+    /* enable logging as specified by global options */
+    if (ctx->g_opt && ctx->g_opt->log_target) {
+        val_log_add_optarg_to_list(&ctx->val_log_targets, ctx->g_opt->log_target, 1);
+    }
+    /* enable logging as specified by dynamic policy */
+    if (ctx->dyn_valpolopt && ctx->dyn_valpolopt->log_target) {
+        val_log_add_optarg_to_list(&ctx->val_log_targets,
+                ctx->dyn_valpolopt->log_target, 1);
     }
-
     /* set the log target from environment */
     logtarget = getenv(VAL_LOG_TARGET);
     if (logtarget) {
         val_log_add_optarg_to_list(&ctx->val_log_targets, logtarget, 1);
     }
 
+    /* 
+     * Merge other dynamic global options into the context 
+     */
+    if (ctx->dyn_valpolopt) {
+        if (VAL_NO_ERROR != 
+                (retval = update_dynamic_gopt(&ctx->g_opt, ctx->dyn_valpolopt)))
+            goto err;
+    }
+
     /* 
      * Free the query cache 
      */
@@ -2001,6 +2150,13 @@ read_res_config_file(val_context_t * ctx)
     struct name_server *ns_head = NULL;
     struct name_server *ns_tail = NULL;
     struct name_server *ns = NULL;
+    unsigned long ns_options = SR_QUERY_RECURSE;
+
+    if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+        ns_options |= SR_QUERY_IPV4_ONLY;
+    } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+        ns_options |= SR_QUERY_IPV6_ONLY;
+    }
 
     while(counter < 255) { /* arbitary way-too-high upper limit */
         /* shouldn't be necessary, but still wise */
@@ -2013,14 +2169,13 @@ read_res_config_file(val_context_t * ctx)
             break;
         }
 
-        ns = parse_name_server(property_buffer, NULL);
+        ns = parse_name_server(property_buffer, NULL, ns_options);
         if (ns == NULL) {
             val_log(ctx, LOG_WARNING,
                     "read_res_config_file(): error parsing android resource!");
             return VAL_CONF_PARSE_ERROR;
         }
 
-        ns->ns_options |= RES_RECURSE;
         if (ns_tail == NULL) {
             ns_head = ns;
             ns_tail = ns;
@@ -2056,11 +2211,11 @@ int
 read_res_config_file(val_context_t * ctx)
 {
     char           *resolv_config;
-    int             fd;
 #ifdef HAVE_FLOCK
     struct flock    fl;
 #endif
     char            token[TOKEN_MAX];
+    int             fd = -1;
     int             line_number = 0;
     int             endst = 0;
     struct name_server *ns_head = NULL;
@@ -2073,11 +2228,31 @@ read_res_config_file(val_context_t * ctx)
     size_t bufsize = 0;
     int retval;
     time_t mtime = 0;
-    
+    unsigned long ns_options = 0;
+
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
 
-    fd = -1;
+    if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+        ns_options |= SR_QUERY_IPV4_ONLY;
+    } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+        ns_options |= SR_QUERY_IPV6_ONLY;
+    }
+    
+    /*
+     * Use any dynamic resolver policy that we may have
+     */
+    if (ctx->dyn_nslist) {
+        clone_ns_list(&ns_head, ctx->dyn_nslist);
+        ns = ns_head;
+        while (ns) {
+            ns_tail = ns;
+            ns = ns->ns_next;
+        }
+    }
+    if (ctx->dyn_polflags & CTX_DYN_POL_RES_OVR) {
+        goto done;
+    }
 
     resolv_config = ctx->resolv_conf;
     if (resolv_config) {
@@ -2090,6 +2265,7 @@ read_res_config_file(val_context_t * ctx)
             FREE(ctx->resolv_conf);
             ctx->resolv_conf = strdup(VAL_DEFAULT_RESOLV_CONF);
             if (ctx->resolv_conf == NULL) {
+                free_name_servers(&ns_head);
                 return VAL_OUT_OF_MEMORY;
             }
             resolv_config = ctx->resolv_conf;
@@ -2114,7 +2290,7 @@ read_res_config_file(val_context_t * ctx)
             goto err;
         } 
         if (0 == sb.st_size)
-            goto empty;
+            goto done;
 
         mtime = sb.st_mtime;
         bufsize = sb.st_size;
@@ -2143,11 +2319,11 @@ read_res_config_file(val_context_t * ctx)
             val_log(ctx, LOG_ERR, "read_res_config_file(): Could not open resolver conf file for reading: %s",
                     resolv_config);
 
-        /* Try to read any inline root.hints information */
+        /* Try to read any inline resolv.conf information */
         mtime = 0;
         bufsize = sizeof(resolv_conf_inline_buf);
         if (!strncmp(resolv_conf_inline_buf, "", sizeof(""))) {
-            goto empty;
+            goto done;
         }
         val_log(ctx, LOG_INFO, 
                 "read_res_config_file(): Using inline resolv.conf data");
@@ -2191,24 +2367,18 @@ read_res_config_file(val_context_t * ctx)
 			"read_res_config_file(): error getting nameserver token!");
                 goto err;
             }
-            if ((ns = parse_name_server(token, DEFAULT_ZONE)) == NULL) {
+            if ((ns = parse_name_server(token, DEFAULT_ZONE, ns_options|SR_QUERY_RECURSE)) == NULL) {
                 val_log(ctx, LOG_WARNING,
-			"read_res_config_file(): error parsing nameserver token!");
+                        "read_res_config_file(): Invalid nameserver addresses '%s'.",
+                        token);
                 goto err;
 	        }
-            if (ns != NULL) {
-                ns->ns_options |= SR_QUERY_RECURSE;
-                if (ns_tail == NULL) {
-                    ns_head = ns;
-                    ns_tail = ns;
-                } else {
-                    ns_tail->ns_next = ns;
-                    ns_tail = ns;
-                }
+            if (ns_tail == NULL) {
+                ns_head = ns;
+                ns_tail = ns;
             } else {
-                val_log(ctx, LOG_WARNING,
-                        "read_res_config_file(): Invalid nameserver addresses '%s', skipping.",
-                        token);
+                ns_tail->ns_next = ns;
+                ns_tail = ns;
             }
         } else if (strncmp(token, "forward", strlen("forward")) == 0) {
 
@@ -2220,24 +2390,31 @@ read_res_config_file(val_context_t * ctx)
                            ALL_COMMENTS, ZONE_END_STMT, 0))) {
                 goto err;
             }
-            if ((ns = parse_name_server(token, DEFAULT_ZONE)) == NULL)
+            if ((ns = parse_name_server(token, DEFAULT_ZONE,
+                            ns_options)) == NULL) {
+                val_log(ctx, LOG_WARNING,
+                        "read_res_config_file(): Invalid nameserver addresses '%s.",
+                        token);
                 goto err;
+            }
+
             /* zone next */
-            if (VAL_NO_ERROR !=
+            if (VAL_NO_ERROR ==
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 0)) &&
+                (ns_name_pton(token, zone_n, sizeof(zone_n)) != -1)) {
+
+                store_ns_for_zone(zone_n, ns);
+
+                free_name_servers(&ns);
+                ns = NULL;
+            } else {
+                free_name_servers(&ns);
+                ns = NULL;
                 goto err;
             }
-            if (ns != NULL) {
-                if (ns_name_pton(token, zone_n, sizeof(zone_n)) == -1)
-                    goto err;
-                store_ns_for_zone(zone_n, ns);
-            } else {
-                val_log(ctx, LOG_WARNING,
-                        "read_res_config_file(): Invalid nameserver addresses '%s', skipping.",
-                        token);
-            }
+
         } else if (strncmp(token, "search", strlen("search")) == 0) {
 
             /* Read the value */
@@ -2255,7 +2432,15 @@ read_res_config_file(val_context_t * ctx)
 
     FREE(buf);
 
-  empty:
+  done:
+
+    if (fd != -1) {
+#ifdef HAVE_FLOCK
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLK, &fl);
+#endif
+        close(fd);
+    }
 
     /*
      * Check if we have root hints 
@@ -2275,13 +2460,6 @@ read_res_config_file(val_context_t * ctx)
     val_log(ctx, LOG_DEBUG, 
             "read_res_config_file(): Done reading resolver configuration");
 
-    if (fd != -1) {
-#ifdef HAVE_FLOCK
-        fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLK, &fl);
-#endif
-        close(fd);
-    }
     return VAL_NO_ERROR;
 
   err:
@@ -2326,7 +2504,7 @@ read_root_hints_file(val_context_t * ctx)
     u_int16_t       type_h, class_h;
     int             success;
     u_long          ttl_h;
-    int             retval;
+    int             retval = VAL_NO_ERROR;
     u_int16_t       rdata_len_h;
     struct rrset_rec *rr_set;
     struct name_server *ns_list = NULL;
@@ -2337,19 +2515,27 @@ read_root_hints_file(val_context_t * ctx)
     char *buf = NULL;
     size_t bufsize = 0;
     time_t mtime;
+    int ipv4_only = 0;
+    int ipv6_only = 0;
 
     class_h = 0;
     have_type = 0;
 
     if (ctx == NULL)
         return VAL_BAD_ARGUMENT;
-    
+   
+   if (val_context_ip4(ctx) && !val_context_ip6(ctx)) {
+       ipv4_only = 1;
+   } else if (!val_context_ip4(ctx) && val_context_ip6(ctx)) {
+       ipv6_only = 1;
+   }
+
     root_hints = ctx->root_conf;
 
     fd = -1;
 
     if (NULL != root_hints && 
-            (fd = open(root_hints, O_RDONLY)) > 0) {
+            (fd = open(root_hints, O_RDONLY)) >= 0) {
 #ifdef HAVE_FLOCK
         memset(&fl, 0, sizeof(fl));
         fl.l_type = F_RDLCK;
@@ -2513,7 +2699,7 @@ read_root_hints_file(val_context_t * ctx)
                        ZONE_COMMENT, ZONE_END_STMT, 0))) {
             goto err;
         }
-        if (type_h == ns_t_a) {
+        if (type_h == ns_t_a && !ipv6_only) {
             struct sockaddr_in sa;
             size_t addrlen4 = sizeof(struct sockaddr_in);
             memset(&sa, 0, sizeof(sa));
@@ -2525,7 +2711,7 @@ read_root_hints_file(val_context_t * ctx)
             rdata_len_h = sizeof(struct in_addr);
             memcpy(rdata_n, &sa.sin_addr, rdata_len_h);
 #ifdef VAL_IPV6
-        } else if (type_h == ns_t_aaaa) {
+        } else if (type_h == ns_t_aaaa && !ipv4_only) {
             struct sockaddr_in6 sa6;
             size_t addrlen6 = sizeof(struct sockaddr_in6);
             memset(&sa6, 0, sizeof(sa6));
@@ -2548,6 +2734,8 @@ read_root_hints_file(val_context_t * ctx)
             continue;
         }
 
+        /* Note that we don't process RRSIGS */
+
         //        SAVE_RR_TO_LIST(NULL, &root_info, zone_n, type_h, type_h, ns_c_in,
         //                        ttl_h, NULL, rdata_n, rdata_len_h, VAL_FROM_UNSET, 0,
         //                        zone_n);
@@ -2558,13 +2746,9 @@ read_root_hints_file(val_context_t * ctx)
             retval = VAL_OUT_OF_MEMORY;
             goto err;
         }
-        if (type_h != ns_t_rrsig) {
-            /* Add this record to its chain. */
-            retval = add_to_set(rr_set, rdata_len_h, rdata_n);
-        } else {
-            /* Add this record's sig to its chain. */
-            retval = add_as_sig(rr_set, rdata_len_h, rdata_n);
-        }
+
+        /* Add this record to its chain. */
+        retval = add_to_set(rr_set, rdata_len_h, rdata_n);
         if (retval != VAL_NO_ERROR) {
             goto err;
         }
@@ -2925,8 +3109,6 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
 
     /* free the policy */
     conf_elem_array[pol->index].free(p);
-    FREE(p);
-    FREE(pol);
     
     /* Flush queries that match this name */
     for(q=ctx->q_list; q; q=q->qc_next) {
@@ -2934,6 +3116,9 @@ val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
             q->qc_flags |= VAL_QUERY_MARK_FOR_DELETION;
         }
     }
+
+    FREE(p);
+    FREE(pol);
     
     retval = VAL_NO_ERROR;
 
