@@ -15,7 +15,7 @@
  * WITH THE USE OR PERFORMANCE OF THE SOFTWARE.
  */
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -55,7 +55,7 @@ static int _process_rcvd_response(val_context_t * context,
  * create a name_server struct from the given address rdata
  */
 static int
-extract_glue_from_rdata(struct val_rr_rec *addr_rr, struct name_server *ns)
+extract_glue_from_rdata(struct rrset_rr *addr_rr, struct name_server *ns)
 {
     struct sockaddr_in *sock_in;
 #ifdef VAL_IPV6
@@ -165,7 +165,7 @@ find_matching_glue(val_context_t *context,
          */
         flags = pc->qc_flags | (VAL_QUERY_GLUE_REQUEST | VAL_QUERY_DONT_VALIDATE);
 
-        if (find_glue_type & Q_WAIT_FOR_A_GLUE) {
+        if (find_glue_type == Q_WAIT_FOR_A_GLUE) {
             glue_type = ns_t_a;
 #ifdef VAL_IPV6
         } else  {/* Q_WAIT_FOR_AAAA_GLUE) */ 
@@ -179,6 +179,7 @@ find_matching_glue(val_context_t *context,
             return retval;
         glueptr = glue_qfq->qfq_query;/* Can never be NULL if glue_qfq is not NULL */
 
+
         /* Add pc and gluptr to our dependency list */
         for (b=*bucket; b; b=b->next_bucket) {
             if (b->qfq == qfq_pc) {
@@ -295,11 +296,16 @@ merge_glue_in_referral(val_context_t *context,
     
     pending_ns = pc->qc_referral->cur_pending_glue_ns;
     if (pending_ns) {
-        if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_A_GLUE, qfq_pc, bucket, queries)))
-            return retval;
 
-        if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_AAAA_GLUE, qfq_pc, bucket, queries)))
-            return retval;
+        if (val_context_ip4(context)) {
+            if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_A_GLUE, qfq_pc, bucket, queries)))
+                return retval;
+        }
+
+        if (val_context_ip6(context)) {
+            if (VAL_NO_ERROR != (retval = find_matching_glue(context, Q_WAIT_FOR_AAAA_GLUE, qfq_pc, bucket, queries)))
+                return retval;
+        }
 
         if (pc->qc_state & Q_WAIT_FOR_GLUE) {
             /* we're not done with fetching glue  */
@@ -385,19 +391,24 @@ merge_glue_in_referral(val_context_t *context,
         flags = pc->qc_flags | 
                 (VAL_QUERY_GLUE_REQUEST | VAL_QUERY_DONT_VALIDATE);
 
-        if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
-                                       queries, pending_ns->ns_name_n, ns_t_a,
-                                       ns_c_in, flags, &added_q)))
-           return retval;
+        pc->qc_state = Q_INIT;
+        if (val_context_ip4(context)) {
+            if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
+                                           queries, pending_ns->ns_name_n, ns_t_a,
+                                           ns_c_in, flags, &added_q)))
+                return retval;
+            pc->qc_state |= Q_WAIT_FOR_A_GLUE;
+        }
 #ifdef VAL_IPV6
-        if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
-                                       queries, pending_ns->ns_name_n, ns_t_aaaa,
-                                       ns_c_in, flags, &added_q)))
-           return retval;
+        if (val_context_ip6(context)) {
+            if (VAL_NO_ERROR != (retval = add_to_qfq_chain(context,
+                                           queries, pending_ns->ns_name_n, ns_t_aaaa,
+                                           ns_c_in, flags, &added_q)))
+                return retval;
+            pc->qc_state |= Q_WAIT_FOR_AAAA_GLUE;
+        }
 #endif
 
-        pc->qc_state = Q_WAIT_FOR_GLUE;
-
     } 
 
     return VAL_NO_ERROR;
@@ -407,7 +418,7 @@ merge_glue_in_referral(val_context_t *context,
  * Check queries in list for missing glue
  * Merge any glue that is available into the relevant query
  * Set *data_missing if some query in the list still remains
- * unanswered
+ * unanswered or received an error response
  */
 int
 fix_glue(val_context_t * context,
@@ -424,9 +435,6 @@ fix_glue(val_context_t * context,
     if (context == NULL || queries == NULL || data_missing == NULL)
         return VAL_BAD_ARGUMENT;
 
-    if (*data_missing == 0)
-        return VAL_NO_ERROR;
-
     *data_missing = 0;
     for (next_q = *queries; next_q; next_q = next_q->qfq_next) {
         /* 
@@ -484,24 +492,24 @@ err:
  * in "pending glue"
  */ 
 int
-res_zi_unverified_ns_list(struct name_server **ns_list,
+res_zi_unverified_ns_list(val_context_t *context,
+                          struct name_server **ns_list,
                           u_char * zone_name,
                           struct rrset_rec *unchecked_zone_info,
                           struct name_server **pending_glue)
 {
     struct rrset_rec *unchecked_set;
-    struct val_rr_rec  *ns_rr;
+    struct rrset_rr  *ns_rr;
     struct name_server *temp_ns;
     struct name_server *ns;
     struct name_server *pending_glue_last;
-    struct name_server *trail_ns;
     struct name_server *outer_trailer;
     struct name_server *tail_ns;
     size_t          name_len;
     int             retval;
     u_char ns_cred = SR_CRED_UNSET;
 
-    if ((ns_list == NULL) || (pending_glue == NULL))
+    if ((context == NULL) || (ns_list == NULL) || (pending_glue == NULL))
         return VAL_BAD_ARGUMENT;
 
     *ns_list = NULL;
@@ -528,8 +536,13 @@ res_zi_unverified_ns_list(struct name_server **ns_list,
                     /*
                      * Create the structure for the name server 
                      */
-                    temp_ns = (struct name_server *)
-                        MALLOC(sizeof(struct name_server));
+                    name_len = wire_name_length(ns_rr->rr_rdata);
+                    if (name_len > NS_MAXCDNAME) {
+                        free_name_servers(ns_list);
+                        *ns_list = NULL;
+                        return VAL_OUT_OF_MEMORY;
+                    }
+                    temp_ns = create_name_server();
                     if (temp_ns == NULL) {
                         /*
                          * Since we're in trouble, free up just in case 
@@ -539,39 +552,18 @@ res_zi_unverified_ns_list(struct name_server **ns_list,
                         return VAL_OUT_OF_MEMORY;
                     }
 
-                    /*
-                     * Make room for the name and insert the name 
-                     */
-                    name_len = wire_name_length(ns_rr->rr_rdata);
-                    if (name_len > sizeof(temp_ns->ns_name_n)) {
-                        free_name_servers(ns_list);
-                        *ns_list = NULL;
-                        return VAL_OUT_OF_MEMORY;
-                    }
                     memcpy(temp_ns->ns_name_n, ns_rr->rr_rdata, name_len);
 
                     /*
                      * Initialize the rest of the fields 
                      */
-                    temp_ns->ns_tsig = NULL;
-                    temp_ns->ns_security_options = ZONE_USE_NOTHING;
                     temp_ns->ns_status = SR_ZI_STATUS_LEARNED;
-
-                    temp_ns->ns_retrans = RES_TIMEOUT;
-                    temp_ns->ns_retry = RES_RETRY;
-
-                    temp_ns->ns_options = SR_QUERY_DEFAULT;
                     /* 
                      * Ensure that recursion is disabled by default 
                      */
                      if (temp_ns->ns_options & SR_QUERY_RECURSE)
                         temp_ns->ns_options ^= SR_QUERY_RECURSE;
 
-                    temp_ns->ns_edns0_size = RES_EDNS0_DEFAULT;
-
-                    temp_ns->ns_next = NULL;
-                    temp_ns->ns_address = NULL;
-                    temp_ns->ns_number_of_addresses = 0;
                     /*
                      * Add the name server record to the list 
                      */
@@ -603,12 +595,11 @@ res_zi_unverified_ns_list(struct name_server **ns_list,
 
     unchecked_set = unchecked_zone_info;
     while (unchecked_set != NULL) {
-        if (unchecked_set->rrs_type_h == ns_t_a ||
-            unchecked_set->rrs_type_h == ns_t_aaaa) {
+        if ((val_context_ip4(context) && unchecked_set->rrs_type_h == ns_t_a) ||
+            (val_context_ip6(context) && unchecked_set->rrs_type_h == ns_t_aaaa)) {
             /*
              * If the owner name matches the name in an *ns_list entry...
              */
-            trail_ns = NULL;
             ns = *ns_list;
             while (ns) {
                 int matching_cred = 1;
@@ -631,7 +622,6 @@ res_zi_unverified_ns_list(struct name_server **ns_list,
                         return retval;
                     break;
                 } else {
-                    trail_ns = ns;
                     ns = ns->ns_next;
                 }
             }
@@ -729,7 +719,7 @@ find_nslist_for_query(val_context_t * context,
         FREE(next_q->qc_zonecut_n);
     next_q->qc_zonecut_n = NULL;
 
-    if (!(next_q->qc_flags & VAL_QUERY_RECURSE) &&
+    if (!(next_q->qc_flags & VAL_QUERY_ITERATE) &&
          context->nslist != NULL) {
         /* 
          * if we have a default name server in our resolv.conf file, send
@@ -757,10 +747,11 @@ find_nslist_for_query(val_context_t * context,
                 "find_nslist_for_query(): Found cached ns_list with cred = %d.", ns_cred);
             /* 
              * If our answer was from an authoritative server, we
-             * also set the flag to denote that we are doing recursion
+             * also set the flag to denote that we are doing an
+             * iterative lookup
              */
             if (ns_cred < SR_CRED_NONAUTH)
-                next_q->qc_flags |= VAL_QUERY_RECURSE;
+                next_q->qc_flags |= VAL_QUERY_ITERATE;
             goto done; 
         } 
     } 
@@ -776,7 +767,7 @@ find_nslist_for_query(val_context_t * context,
                 "find_nslist_for_query(): Trying to answer query recursively, but no root hints file found.");
         return VAL_CONF_NOT_FOUND;
     }
-    next_q->qc_flags |= VAL_QUERY_RECURSE;
+    next_q->qc_flags |= VAL_QUERY_ITERATE;
     clone_ns_list(&next_q->qc_ns_list, context->root_ns);
     next_q->qc_zonecut_n = (u_char *) MALLOC(sizeof(u_char));
     if (next_q->qc_zonecut_n == NULL) {
@@ -845,12 +836,12 @@ bootstrap_referral(val_context_t *context,
         }
         clone_ns_list(ref_ns_list, context->root_ns);
         matched_q->qc_state = Q_INIT;
-        matched_q->qc_flags |= VAL_QUERY_RECURSE;
+        matched_q->qc_flags |= VAL_QUERY_ITERATE;
         return VAL_NO_ERROR;
     }
     
     if ((ret_val =
-         res_zi_unverified_ns_list(ref_ns_list, referral_zone_n,
+         res_zi_unverified_ns_list(context, ref_ns_list, referral_zone_n,
                                    learned_zones, &pending_glue))
         != VAL_NO_ERROR) {
         return ret_val;
@@ -873,6 +864,12 @@ bootstrap_referral(val_context_t *context,
          */
         if (matched_q->qc_referral == NULL) {
             ALLOCATE_REFERRAL_BLOCK(matched_q->qc_referral);
+            if (matched_q->qc_referral == NULL) {
+                free_name_servers(&pending_glue);
+                free_name_servers(ref_ns_list);
+                *ref_ns_list = NULL;
+                return VAL_OUT_OF_MEMORY;
+            }
         } 
             
         matched_q->qc_referral->cur_pending_glue_ns = NULL;
@@ -894,33 +891,56 @@ bootstrap_referral(val_context_t *context,
                     len);
         }
 
+
         if (*ref_ns_list != NULL) {
+
+            /* save the pending list for some future occasion */
             matched_q->qc_referral->pending_glue_ns = pending_glue;
             matched_q->qc_state = Q_INIT;
+
+        } else if (!namecmp(pending_glue->ns_name_n, matched_q->qc_name_n)) {
+
+            /* 
+             * Break out of a cyclic dependency
+             * Ideally we want the next NS higher up, but its not worth
+             * the trouble. Simply start from root in such circumstances
+             */
+            free_name_servers(&pending_glue);
+            if (context->root_ns != NULL) {
+                clone_ns_list(ref_ns_list, context->root_ns);
+                matched_q->qc_flags |= VAL_QUERY_ITERATE;
+                matched_q->qc_state = Q_INIT;
+            }
+
         } else {
+
             matched_q->qc_referral->cur_pending_glue_ns = pending_glue;
             matched_q->qc_referral->pending_glue_ns = pending_glue->ns_next;
             pending_glue->ns_next = NULL;
 
             /*
              * Create a query for glue for pending_ns 
-            */
-            flags = matched_q->qc_flags | 
+             */
+            flags = matched_q->qc_flags | VAL_QUERY_ITERATE | 
                         (VAL_QUERY_GLUE_REQUEST | VAL_QUERY_DONT_VALIDATE);
+            matched_q->qc_state = Q_INIT;
 
-            matched_q->qc_state = Q_WAIT_FOR_A_GLUE;
-            if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
+            if (val_context_ip4(context)) {
+                matched_q->qc_state |= Q_WAIT_FOR_A_GLUE;
+                if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
                                        queries, pending_glue->ns_name_n, ns_t_a,
                                        ns_c_in, flags, &added_q)))
-                return ret_val;
+                    return ret_val;
+            }
 #ifdef VAL_IPV6
-            matched_q->qc_state |= Q_WAIT_FOR_AAAA_GLUE;
-            if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
+            if (val_context_ip6(context)) {
+                matched_q->qc_state |= Q_WAIT_FOR_AAAA_GLUE;
+                if (VAL_NO_ERROR != (ret_val = add_to_qfq_chain(context,
                                        queries, pending_glue->ns_name_n, ns_t_aaaa,
                                        ns_c_in, flags, &added_q)))
-                return ret_val;
+                    return ret_val;
+            }
 #endif
-
         }
     } else if (*ref_ns_list != NULL) {
         matched_q->qc_state = Q_INIT;
@@ -1019,6 +1039,9 @@ follow_referral_or_alias_link(val_context_t * context,
 
     if (matched_q->qc_referral == NULL) {
         ALLOCATE_REFERRAL_BLOCK(matched_q->qc_referral);
+        if (matched_q->qc_referral == NULL) {
+            return VAL_OUT_OF_MEMORY;
+        }
     } else if (alias_chain && matched_q->qc_referral->queries) {
         /* free up the old set of registered queries and start afresh */
         deregister_queries(&matched_q->qc_referral->queries);
@@ -1032,7 +1055,8 @@ follow_referral_or_alias_link(val_context_t * context,
         matched_q->qc_referral->qnames = *qnames;
     else if (*qnames) {
         struct qname_chain *t_q;
-        for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next);
+        for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next)
+            ; /* no body. ';' on new line to suppress warning. */
         t_q->qnc_next = matched_q->qc_referral->qnames;
         matched_q->qc_referral->qnames = *qnames;
     }
@@ -1060,7 +1084,7 @@ follow_referral_or_alias_link(val_context_t * context,
         return VAL_NO_ERROR;
     } 
 
-    matched_q->qc_flags |= VAL_QUERY_RECURSE;
+    matched_q->qc_flags |= VAL_QUERY_ITERATE;
 
     res_sq_free_rrset_recs(proofs);
     *proofs = NULL;
@@ -1112,6 +1136,7 @@ follow_referral_or_alias_link(val_context_t * context,
     } else if (VAL_NO_ERROR != (ret_val = stow_zone_info(learned_zones, matched_q))) {
         res_sq_free_rrset_recs(learned_zones);
         *learned_zones = NULL;
+        free_name_servers(&ref_ns_list);
         return ret_val;
     }
     *learned_zones = NULL; /* consumed */
@@ -1202,26 +1227,20 @@ follow_referral_or_alias_link(val_context_t * context,
         struct rrset_rec *rr_set;                                       \
         int ret_val;                                                    \
         u_char *r;                                                      \
-        if (authoritive)                                                \
-            rr_set = find_rr_set (respondent_server, listtype, name_n,  \
+        rr_set = find_rr_set (respondent_server, listtype, name_n,  \
                               type_h, set_type_h, class_h, ttl_h, hptr, \
                               rdata, from_section, authoritive,         \
                               iterative, zonecut_n);                    \
-        else                                                            \
-            rr_set = find_rr_set (respondent_server, listtype, name_n,  \
-                              type_h, set_type_h, class_h, ttl_h, hptr, \
-                              rdata, from_section, authoritive,         \
-                              iterative, NULL);                         \
         if (rr_set==NULL) {                                             \
             ret_val = VAL_OUT_OF_MEMORY;                                \
         }                                                               \
         else {                                                          \
             if (type_h != ns_t_rrsig) {                                 \
-                /* Add this record to its chain of val_rr_rec's. */     \
+                /* Add this record to its chain. */                     \
                 ret_val = add_to_set(rr_set,rdata_len_h,rdata);         \
             } else if (VAL_NO_ERROR ==                                  \
                     (ret_val = add_as_sig(rr_set,rdata_len_h,rdata))) { \
-                /* Add this record to the sig of rrset_rec (above). */  \
+                /* Add this record's sig to its chain. */               \
                 /* and override the zonecut using the rrsig info */     \
                 if (rr_set->rrs_zonecut_n) {                            \
                     FREE(rr_set->rrs_zonecut_n);                        \
@@ -1314,8 +1333,12 @@ process_cname_dname_responses(u_char *name_n,
          */
         if ((ret_val = add_to_qname_chain(qnames, rdata)) != VAL_NO_ERROR)
             return ret_val; 
-        if (!matched_q->qc_referral)
+        if (!matched_q->qc_referral) {
             ALLOCATE_REFERRAL_BLOCK(matched_q->qc_referral);
+            if (matched_q->qc_referral == NULL) {
+                return VAL_OUT_OF_MEMORY;
+            }
+        }
             
         if (register_query(&matched_q->qc_referral->queries,
                            rdata,
@@ -1354,8 +1377,12 @@ process_cname_dname_responses(u_char *name_n,
              */
             if ((ret_val = add_to_qname_chain(qnames, name_n)) != VAL_NO_ERROR)
                 return ret_val; 
-            if (!matched_q->qc_referral)
+            if (!matched_q->qc_referral) {
                 ALLOCATE_REFERRAL_BLOCK(matched_q->qc_referral);
+                if (matched_q->qc_referral == NULL) {
+                    return VAL_OUT_OF_MEMORY;
+                }
+            }
 
             if (register_query(&matched_q->qc_referral->queries,
                                name_n,
@@ -1423,6 +1450,7 @@ prepare_empty_nonexistence(struct rrset_rec **answers,
     (*answers)->rrs_type_h = query_type_h;
     (*answers)->rrs_class_h = query_class_h;
     (*answers)->rrs_ttl_h = 0;/* don't have any basis to set the TTL value */
+    (*answers)->rrs_ttl_x = 0;
     (*answers)->rrs_cred = SR_CRED_UNSET;
     (*answers)->rrs_section = VAL_FROM_UNSET;
     if ((respondent_server) &&
@@ -1479,8 +1507,9 @@ static void consume_referral_data(struct delegation_info **qc_referral,
         *qnames = (*qc_referral)->qnames;
     else if ((*qc_referral)->qnames) {
         struct qname_chain *t_q;
-        for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next);
-            t_q->qnc_next = (*qc_referral)->qnames;
+        for (t_q = *qnames; t_q->qnc_next; t_q = t_q->qnc_next)
+            ;
+        t_q->qnc_next = (*qc_referral)->qnames;
     }
     (*qc_referral)->qnames = NULL;
 
@@ -1512,7 +1541,7 @@ digest_response(val_context_t * context,
                 size_t response_length, 
                 struct domain_info *di_response)
 {
-    u_int16_t       question, answer, authority, additional;
+    u_int16_t       answer, authority, additional;
     u_int16_t       rrs_to_go;
     int             i;
     size_t          response_index;
@@ -1551,7 +1580,6 @@ digest_response(val_context_t * context,
     struct val_query_chain *matched_q;
     char query_name_p[NS_MAXDNAME];
     char rrs_zonecut_p[NS_MAXDNAME];
-    char name_p[NS_MAXDNAME];
     struct name_server *resp_ns = NULL;
     int isrelv = 0;
     char name_buf[INET6_ADDRSTRLEN + 1];
@@ -1576,7 +1604,6 @@ digest_response(val_context_t * context,
     hptr = NULL;
     rdata = NULL;
 
-    question = ntohs(header->qdcount);
     answer = ntohs(header->ancount);
     authority = ntohs(header->nscount);
     additional = ntohs(header->arcount);
@@ -1726,20 +1753,20 @@ digest_response(val_context_t * context,
          */
         qc = *qnames;
         if (qc &&
-              (!namecmp(qc->qnc_name_n, name_n)) || 
+              ((!namecmp(qc->qnc_name_n, name_n)) || 
               ((set_type_h == ns_t_dname) &&
                     namename(qc->qnc_name_n, name_n)) ||
               ((type_h == ns_t_rrsig) &&
-                    name_in_qname_chain(qc, name_n))) {
+                    name_in_qname_chain(qc, name_n)))) {
             isrelv = 1;
         } else {
             isrelv = 0;
         }
 
-        authoritive = (matched_q->qc_flags & VAL_QUERY_RECURSE) &&
+        authoritive = (matched_q->qc_flags & VAL_QUERY_ITERATE) &&
                       (header->aa == 1);
 
-        iterative = matched_q->qc_flags & VAL_QUERY_RECURSE;
+        iterative = matched_q->qc_flags & VAL_QUERY_ITERATE;
         /*
          * If it is from the answer section, it may be an alias 
          * If it is from the authority section, it may be a proof or a referral 
@@ -1750,8 +1777,39 @@ digest_response(val_context_t * context,
 
             if (nothing_other_than_alias) {
                 /*
-                 * check if we received a non-alias or had explicitly asked for this alias 
+                 * check if we received an alias and had not explicitly
+                 * asked for it 
                  */
+                if (((set_type_h == ns_t_cname && query_type_h != ns_t_cname) || 
+                     (set_type_h == ns_t_dname && query_type_h != ns_t_dname)) &&
+                    (ALIAS_MATCH_TYPE(query_type_h))) {
+
+                    int referral_error = 0;
+                    /* process CNAMEs or DNAMEs if they exist */
+                    if ((VAL_NO_ERROR != (ret_val = 
+                            process_cname_dname_responses(name_n, type_h, rdata, 
+                                                  matched_q, qnames, 
+                                                  &referral_error))) || 
+                            (referral_error)) {
+                        if (referral_error) 
+                            val_log(context, LOG_DEBUG, "digest_response(): CNAME/DNAME error or loop encountered");
+                        goto done;
+                    }
+                    /* forget the current zonecut */
+                    if (matched_q->qc_zonecut_n) {
+                        FREE(matched_q->qc_zonecut_n);
+                        matched_q->qc_zonecut_n = NULL;
+                        rrs_zonecut_n = NULL; 
+                    }
+                } else {
+                    /* 
+                     * We've presumably reached targets of the alias. 
+                     * Don't follow aliases For types that cannot be aliased
+                     */
+                    nothing_other_than_alias = 0;
+                }
+
+#if 0
                 if ((set_type_h != ns_t_cname && set_type_h != ns_t_dname) ||
                     ((query_type_h == ns_t_cname) && (set_type_h == ns_t_cname)) ||
                     ((query_type_h == ns_t_dname) && (set_type_h == ns_t_dname)) ||
@@ -1780,6 +1838,7 @@ digest_response(val_context_t * context,
                 } else {
                     nothing_other_than_alias = 0;
                 }
+#endif
             }
             SAVE_RR_TO_LIST(resp_ns, 
                             &learned_answers, name_n, type_h,
@@ -1789,20 +1848,25 @@ digest_response(val_context_t * context,
         } else if (from_section == VAL_FROM_AUTHORITY) {
             if ((set_type_h == ns_t_nsec)
 #ifdef LIBVAL_NSEC3
-                    || (set_type_h == ns_t_nsec3)
+                || (set_type_h == ns_t_nsec3)
 #endif
-                    || (set_type_h == ns_t_soa)) {
-
+               ) {
                 proof_seen = 1;
-                if (set_type_h == ns_t_soa) {
-                    soa_seen = 1;
-                }
-
                 SAVE_RR_TO_LIST(resp_ns, 
                                 &learned_proofs, name_n, type_h,
                                 set_type_h, class_h, ttl_h, hptr, rdata,
                                 rdata_len_h, from_section, authoritive,
                                 iterative, rrs_zonecut_n);
+
+            } else if (set_type_h == ns_t_soa) {
+
+                proof_seen = 1;
+                soa_seen = 1;
+                SAVE_RR_TO_LIST(resp_ns, 
+                                &learned_proofs, name_n, type_h,
+                                set_type_h, class_h, ttl_h, hptr, rdata,
+                                rdata_len_h, from_section, authoritive,
+                                iterative, name_n);
             } else if (set_type_h == ns_t_ns) {
                 /* 
                  * The zonecut information for name servers is 
@@ -1827,7 +1891,8 @@ digest_response(val_context_t * context,
                                 type_h, set_type_h, class_h, ttl_h, hptr,
                                 rdata, rdata_len_h, from_section,
                                 authoritive, iterative, rrs_zonecut_n);
-            } else if (set_type_h == ns_t_a || set_type_h == ns_t_aaaa) {
+            } else if ((val_context_ip4(context) && set_type_h == ns_t_a) || 
+                       (val_context_ip6(context) && set_type_h == ns_t_aaaa)) {
                 SAVE_RR_TO_LIST(resp_ns,
                                 &learned_zones, name_n,
                                 type_h, set_type_h, class_h, ttl_h, hptr,
@@ -2129,6 +2194,7 @@ val_resquery_send(val_context_t * context,
     struct name_server *tempns;
     struct val_query_chain *matched_q;
     struct name_server *nslist;
+    struct timeval now;
 
     val_log(NULL, LOG_DEBUG, __FUNCTION__);
     /*
@@ -2139,7 +2205,7 @@ val_resquery_send(val_context_t * context,
     if ((matched_qfq == NULL) || 
         (matched_qfq->qfq_query->qc_ns_list == NULL)
 #ifndef VAL_NO_ASYNC
-        || (matched_qfq->qfq_flags & VAL_QUERY_ASYNC)
+        || (matched_qfq->qfq_query->qc_flags & VAL_QUERY_ASYNC)
 #endif
         ) {
         return VAL_BAD_ARGUMENT;
@@ -2168,6 +2234,12 @@ val_resquery_send(val_context_t * context,
         }
     }
 
+    /*
+     * Update the qc_last_sent timestamp
+     */
+    gettimeofday(&now, NULL);
+    matched_q->qc_last_sent = now.tv_sec;
+
     if ((ret_val =
          query_send(name_p, matched_q->qc_type_h, matched_q->qc_class_h,
                     nslist, &(matched_q->qc_trans_id))) == SR_UNSET)
@@ -2204,7 +2276,7 @@ val_resquery_rcv(val_context_t * context,
     if ((matched_qfq == NULL) || (response == NULL) || (queries == NULL) ||
         (pending_desc == NULL)
 #ifndef VAL_NO_ASYNC
-        || (matched_qfq->qfq_flags & VAL_QUERY_ASYNC)
+        || (matched_qfq->qfq_query->qc_flags & VAL_QUERY_ASYNC)
 #endif
         )
         return VAL_BAD_ARGUMENT;
@@ -2296,9 +2368,8 @@ val_res_nsfallback(val_context_t *context, struct val_query_chain *matched_q,
                 "val_res_nsfallback(): Doing EDNS0 fallback"); 
     }
     else {
-        matched_q->qc_state = Q_RESPONSE_ERROR;
         val_log(context, LOG_DEBUG,
-                "val_res_nsfallback(): EDNS0 fallback failed"); 
+                "val_res_nsfallback(): Moving to next address"); 
     }
 }
 
@@ -2525,15 +2596,15 @@ val_async_select_info(val_context_t *ctx, fd_set *activefds,
         return VAL_BAD_ARGUMENT;
 
     val_log(NULL, LOG_DEBUG, __FUNCTION__);
+    gettimeofday(&now, NULL);
 
     /** need to adjust relative timeout to absolute time used by libval */
     if (timeout) {
         if(timeout->tv_sec < LONG_MAX) {
             /* add current time to delay */
-            gettimeofday(&now, NULL);
             timeradd(&now, timeout, &closest);
         } else
-            memcpy(closest_event, timeout, sizeof(closest_event));
+            memcpy(closest_event, timeout, sizeof(struct timeval));
         if (closest.tv_sec < 0) {
             closest.tv_sec = 0;
             closest.tv_usec = 0;
@@ -2547,6 +2618,8 @@ val_async_select_info(val_context_t *ctx, fd_set *activefds,
 
     for (as = context->as_list; as; as = as->val_as_next) {
 
+        int cache_only = 1;
+
 #ifndef VAL_NO_THREADS
         if (! (as->val_as_ctx->ctx_flags & CTX_PROCESS_ALL_THREADS) &&
             (! pthread_equal(self, as->val_as_tid)))
@@ -2562,7 +2635,7 @@ val_async_select_info(val_context_t *ctx, fd_set *activefds,
             char         name_p[NS_MAXDNAME];
             if (-1 == ns_name_ntop(qfq->qfq_query->qc_name_n, name_p, sizeof(name_p)))
                 snprintf(name_p, sizeof(name_p), "unknown/error");
-            if (!qfq->qfq_query->qc_ea) {
+            if (!qfq->qfq_query->qc_ea || (qfq->qfq_query->qc_flags & VAL_QUERY_SKIP_RESOLVER)) {
                 val_log(NULL, LOG_DEBUG+1, " as %p query %p {%s %s(%d) %s(%d)} ea %p", as, qfq,
                         name_p, p_class(qfq->qfq_query->qc_class_h),
                         qfq->qfq_query->qc_class_h,
@@ -2570,6 +2643,7 @@ val_async_select_info(val_context_t *ctx, fd_set *activefds,
                         qfq->qfq_query->qc_type_h, qfq->qfq_query->qc_ea);
                 continue;
             }
+            cache_only = 0;
             val_log(NULL, LOG_DEBUG, " as %p query %p {%s %s(%d) %s(%d)} ea %p", as, qfq,
                     name_p, p_class(qfq->qfq_query->qc_class_h),
                     qfq->qfq_query->qc_class_h,
@@ -2578,6 +2652,10 @@ val_async_select_info(val_context_t *ctx, fd_set *activefds,
             res_async_query_select_info(qfq->qfq_query->qc_ea, nfds, activefds,
                                         closest_event);
         }
+        if (cache_only) {
+            closest.tv_sec = 0;
+            closest.tv_usec = 0;
+        }
     }
 
     CTX_UNLOCK_ACACHE(context);
